/*
 Copyright (C) 2010-2017 by Hoefer Alfred

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public License
 as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version. 

 This library is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 See the GNU Lesser General Public License for more details. 

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free
 Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110, USA
*/

#ifndef	__TRS232ALD_H__
#define	__TRS232ALD_H__    1

#include "trs232.h"

#define RSBEF_SYNC		0x01
#define RSBEF_OK		0x02
#define RSBEF_ERROR		0x03
#define RSBEF_GET_INFO		0x04
#define RSBEF_GET_STATUS	0x05
#define RSBEF_READ_DATA		0x08
#define RSBEF_READ_DATA_4X	0x09
#define RSBEF_READ_DATA_TYPE	0x0a
#define RSBEF_READ_DATA_TYPE_4X	0x0b
#define RSBEF_WRITE_DATA	0x0c
#define RSBEF_BRIDGE_MODE	0x0d

#define RSBEF_I2C_BRIDGE	0x10
#define RSBEF_SPI_BRIDGE	0x11


#define RSBEF_USER		0x40
#define RSBEF_USER_MAX		0x7f

#define RSBEF_4xMODE		0x100

#define CTL_TOGGLE		0x10
#define CTL_4x			0x20

#define RSERR_CRC		0x01   /* Error summe (repeat data max. 6x) */
#define RSERR_BUFLEN		0x02   /* Error Buffer length */
#define RSERR_UNKNOWN_BEF	0x03
#define RSERR_BEF_SIZE		0x04
#define RSERR_ADDR_FAULT	0x05
#define RSERR_LEN_OVERFLOW	0x06
#define RSERR_READ_ONLY		0x07
#define RSERR_NO_BRIDGE_MODE	0x08

/* ----------------- Read Data type ---------------------- */
#define RSDT_ABSOLUTE		0x00
#define RSDT_FLASH		0x01
#define RSDT_EEPROM		0x02
#define RSDT_RAM		0x03
#define RSDT_SFR		0x04
#define RSDT_CRT_ASCII		0x05
#define RSDT_CRT_ASCII_COL	0x06
#define RSDT_CRT_GRAFIK		0x07


/* -------------------- Info 0 --------------------------- */
#define HW_TYPE_NONE		0
#define HW_TYPE_PROGRAMMER	0xfc

#define FACTORY_MICROCHIP	1

#define CPUMICROCHIP_dsPIC33FJ64MC506	0x0089
#define CPUMICROCHIP_dsPIC33FJ128MC802	0x0629
#define CPUMICROCHIP_dsPIC33EP128GP502	0x1E4D
#define CPUMICROCHIP_dsPIC33EP128GP506	0x1E4F

#define BUF_SIZE_64		0
#define BUF_SIZE_128		1
#define BUF_SIZE_192		2
#define BUF_SIZE_256		3
#define BUF_SIZE_512		5
#define BUF_SIZE_MASK		0x0007

/* ----------------------------------------------------------------------

  +-----+-----+-----+-----+----------------+-----+
  |  ff | Bef | len | ctl |     Data       | sum |
  +-----+-----+-----+-----+----------------+-----+
     |     |     |     |    0-253(512) Byte   |
     |     |     |     |                      +-- 0x39 + Bef + len + ctl + data
     |     |     |     +-- b0..3 = ((Bef+len) ^ ~((Bef+len+ctl)>>4)) & 0x0f
     |     |     |         b4 toggle bit
     |     |     |         b5 Data len 4x (max. 512 Byte)
     |     |     |         b6..7 Data compress  0=no 1=RLE 2=LZW 3=LZO
     |     |     |         
     |     |     +-- data length 0..252 (<252 = Error)         
     |     +-- 1..254 (0x00 and 0xff = Error)
     +-- Start Byte

Bef 0x01 Sync	clear toggle bit
		len=0
	Return: Ok
Bef 0x02 Ok	only from slave
		len=0
Bef 0x03 Error
		len=1 -> protocol and system Error
			data0:
			0x01 = Error summe (repeat data max. 6x)
			0x02 = Error Buffer length
		len >= 2 User Error
Bef 0x04 Get Info
		len=1	1 Byte info type
	info type 0 = Return:
		Byte 0: HW Type
			0xfc = Programmer
		Byte 1: CPU Factory  1=Microchip
		Byte 2,3: CPU Type
		Byte 4,5: CPU Ver

		Byte 6,7: Data art
		6.0 .. 6.2  max. data buffer	0=64  1=128  2=192  3=256 
						4=--- 5=512  6=---  7=---
		6.3	has 4x modus
		6.4	
		6.5	has RLE
		6.6	has LZW
		6.7	has LZO

		7.0	absolute address
		7.1	flash
		7.2	eeprom
		7.3	ram
		7.4	sfr
		Byte 8: CRT
		8.0	crt ram ASCII
		8.1	crt ram ASCII + Color
		8.2	has UTF8
		8.3	crt ram Grafik
		Byte 9  crt bit 0 = ASCII  1..32 Color bit
		Byte 10..11    crt width
		Byte 12..13  crt height
		Byte 14..15  Extra  (if len==14 --> Set Extra=0)

Bef 0x05 Get Status
		len=1	1 Byte status type

Bef 0x08 Read Data
		len=4	3 Byte address + 1 Byte len 0-254
		len=5	4 Byte address + 1 Byte len 0-254
Bef 0x09 Read Data 4x
		len=4	3 Byte address + 1 Byte 4x len 0-128
		len=5	4 Byte address + 1 Byte 4x len 0-128

Bef 0x0a Read Data type
		len=5	1 Byte type + 3 Byte address + 1 Byte len 0-254
		len=6	1 Byte type + 4 Byte address + 1 Byte len 0-254
Bef 0x0b Read Data type 4x
		len=5	1 Byte type + 3 Byte address + 1 Byte 4x len 0-128
		len=6	1 Byte type + 4 Byte address + 1 Byte 4x len 0-128
	Read Data type:
			0x00 = absolute address
			0x01 = flash
			0x02 = eeprom
			0x03 = ram
			0x04 = sfr
			0x05 = crt ram ASCII
			0x06 = crt ram ASCII + Color
			0x07 = crt ram Grafik

Bef 0x0c Write Data     addr=4Byte
		len=4+data	4 Byte address + Data
Bef 0x0d Write Data 4x  addr=4Byte
		len=4+data	4 Byte address + Data
---------------------------------------------------------------------- */

typedef struct {
  byte HW_Type;
  byte CPU_Factory;
  word CPU_Type;
  word CPU_Ver;
  word DataArt;
  byte CRT;
  byte CRT_Color;
  word CRT_Width;
  word CRT_Height;
  word Extra;
} __attribute__((packed)) TSysInfo0;

#define BSI0_CRT_ASCII		1
#define BSI0_CRT_ASCII_COL	(1<<1)
#define BSI0_CRT_ASCII_UTF8	(1<<2)
#define BSI0_CRT_GRAFIK		(1<<3)
/* TSysInfo0.Extra */
#define BSI0EX_BRIDGE_I2C_MASK		0x03
#define BSI0EX_BRIDGE_I2C_16B		0x01
#define BSI0EX_BRIDGE_I2C_32B		0x02
#define BSI0EX_BRIDGE_I2C_64B		0x03

#define BSI0EX_BRIDGE_SPI	(1<<2)


#define AlgTrs232ald    dword TxD_tgl; \
  dword LastTxDBef; \
  void (*OnTimeOut)(struct TCSelf *self); \
  void (*OnTimeOutCount)(struct TCSelf *self, int count); \
  void (*OnRxDData)(struct TCSelf *self, dword bef, const byte *data, size_t lg);

#define TCSelf Trs232ald
typedef struct Trs232ald {
  AlgTrs232
  AlgTrs232ald
} Trs232ald;
#undef TCSelf

extern void *Trs232ald_Create_struct_init(int tcsize, int wtyp);
extern Trs232ald *Trs232ald_Create();
extern bool Trs232ald__Destroy(Trs232ald *self);

extern bool rs232ald_SendData(Trs232ald *self, unsigned int bef, 
				const byte *data, size_t lg);

extern const char *Trs232ald_GetErrorStr(dword orig_bef, dword bef, 
				const byte *data, size_t lg, bool unknown);



#endif /* __TRS232ALD_H__ */
