/*
 Copyright (C) 2010-2020 by Hoefer Alfred

 Licensed under the LGPL v2.1, see the file COPYING.LIB
*/

#include "sysbase.h"
#include "jpeg_pv.h"

#if LD_LIB_JPEG

#define dl_jpeg_std_error jpeg_std_error

#else /* LD_LIB_JPEG */

#include <dlfcn.h>

void *jpegLibHandle = NULL;
int jpg_ver;
size_t jpg_size_decompress;
size_t jpg_size_compress;


bool jpeglib_RD_load = false;
bool jpeglib_WR_load = false;
bool jpeglib_Err = false;

struct jpeg_error_mgr *(*dl_jpeg_std_error)(struct jpeg_error_mgr * err);
void (*dl_jpeg_CreateDecompress)(j_decompress_ptr cinfo,
					int version, size_t structsize);
#endif /* LD_LIB_JPEG == 0 */

struct my_error_mgr {
  struct jpeg_error_mgr pub;	/* "public" fields */
  jmp_buf setjmp_buffer;	/* for return to caller */
};

static void __attribute__ ((noreturn)) jpeg_error_exit(j_common_ptr cinfo) {
  /* cinfo->err really points to a my_error_mgr struct, so coerce pointer */
  my_error_ptr myerr = (my_error_ptr) cinfo->err;

  /* Always display the message. */
  /* We could postpone this until after returning, if we chose. */
  (*cinfo->err->output_message) (cinfo);

  /* Return control to the setjmp point */
  longjmp(myerr->setjmp_buffer, 1);
}

void REGP2 jpeg_error_init(j_common_ptr cinfo, my_error_ptr jerr) {
	cinfo->err = dl_jpeg_std_error(&jerr->pub);
	jerr->pub.error_exit = jpeg_error_exit;
}

#if LD_LIB_JPEG == 0

typedef struct {
  const char *so;
  const int ver;
  const size_t size_decompress;
  const size_t size_compress;
} TTabJPEG;

static void CHK_JPEG_Ver(const TTabJPEG *tab) {
  struct jpeg_decompress_struct *cinfo;
  struct my_error_mgr jerr;

	if (!tab || !tab->so) return;
	jpegLibHandle = dlopen(tab->so, RTLD_LAZY);
	if (!jpegLibHandle) return;

	dl_jpeg_std_error = load_lib(jpegLibHandle, "jpeg_std_error");
	dl_jpeg_CreateDecompress = load_lib(jpegLibHandle, "jpeg_CreateDecompress");

	if (!dl_jpeg_std_error || !dl_jpeg_CreateDecompress) goto err;
// printf("%s(%d): sizeof jpeg_decompress_struct = %d\n", FL, sizeof(struct jpeg_decompress_struct));
// printf("%s(%d): sizeof jpeg_compress_struct   = %d\n", FL, sizeof(struct jpeg_compress_struct));
	cinfo = xmalloc(tab->size_decompress);
	jpeg_error_init((j_common_ptr)cinfo, &jerr);
	if (setjmp(jerr.setjmp_buffer)) {
	    xfree(cinfo);
err:
	    dlclose(jpegLibHandle);
	    jpegLibHandle = NULL;
	    return;
	}
	dl_jpeg_CreateDecompress(cinfo, tab->ver, tab->size_decompress);
	xfree(cinfo);

	jpg_ver = tab->ver;
	jpg_size_decompress = tab->size_decompress;
	jpg_size_compress = tab->size_compress;
}


#if defined (__i386__)
static const TTabJPEG TabJPEG[] = {
  {"libjpeg.so.62", 62, 464, 372},
  {"libjpeg.so.7", 70, 468, 416},
  {"libjpeg.so.8", 80, 484, 428},
  {NULL, 0}
};
#elif defined(__x86_64__)
static const TTabJPEG TabJPEG[] = {
  {"libjpeg.so.62", 62, 632, 520},
  {"libjpeg.so.8", 80, 656, 584},
  {"libjpeg.so.7", 70, 640, 568},
  {NULL, 0}
};
#else
# error "libjpeg dyn-load Unsupported architecture!"
#endif

bool LoadLib_JPEG() {
  const TTabJPEG *tab;
  char *pc;

	if (jpeglib_Err) return false;
	if (jpegLibHandle) return true;

	tab = TabJPEG;
	while (!jpegLibHandle && tab->so) {
	    CHK_JPEG_Ver(tab);
	    tab++;
	}

	jpeglib_Err = (jpegLibHandle == NULL);
	if (jpeglib_Err) {
	    pc = pcdup("can't open ");
	    tab = TabJPEG;
	    int i = 3;
	    while (tab->so) {
		ppcadd(&pc, tab->so);
		tab++;
		i--;
		if (tab->so) {
		    ppcadd(&pc, " or ");
		    if (!i) {
			i = 3;
			ppcadd(&pc, "\n               ");
		    }
		}
	    }
	    appError(EMSG_ERR2, pc);
	    xfree(pc);
	}

	return !jpeglib_Err;
}
#endif /* LD_LIB_JPEG == 0 */
