/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 Licensed under the LGPL v2.1, see the file COPYING.LIB
*/

#include "image.h"
/*
https://de.wikipedia.org/wiki/Portable_Anymap

Magic Number	Dateityp	Kodierung
P1	Portable Bitmap		ASCII
P2	Portable Graymap	ASCII
P3	Portable Pixmap		ASCII
P4	Portable Bitmap		Bin
P5	Portable Graymap	Bin
P6	Portable Pixmap		Bin

Portable Bitmap  *.pbm	0 und 1				1 Bit
Portable Graymap *.pgm	0 ... 255 bzw. 0 ... 65535	8 Bit bzw. 16 Bit
Portable Pixmap  *.ppm	0 ... 255 bzw. 0 ... 65535	24 bzw. 48 Bit

--- Only *.pam (Netpbm Formats) save ALPHA ----
P7
WIDTH 227
HEIGHT 149
DEPTH 3
MAXVAL 255
TUPLTYPE RGB
ENDHDR

*/

static char *wr_colval(char *pc, int value) {
	ppcmove_v32(&pc, value, kSPACE_ADD);
	return pc;
}

static void write_fb(TImage *self, char *pc) {
	fwrite(self->tIO, 1, pc - (char*)self->tIO, self->out_fp);
}

#define PNM_P1 1
#define PNM_P2 2
#define PNM_P3 3
#define PNM_P4 4
#define PNM_P5 5
#define PNM_P6 6
#define PNM_P7 7

int write_pnm(TImage *self) {
  int m, ch, pnmType, ColCount, iobsize, iobidx;
  char *buf, *pc, *iobmem;
  const char *cpc;
  dword A;
  bool ALPHA;
  bool TxtPlain;
  bool rgb565, is_pbm, is_pgm;

	TxtPlain = (self->out_fu & fuWR_txtplain) != 0;
	ALPHA = false;
	is_pbm = false;
	is_pgm = false;
	pnmType = 0;
	cpc = GetFileExt_pc(self->out_gp->name);
	if (!pccasecmpASCII(cpc, "pbm")) {
	    pnmType = PNM_P4;
	    if (TxtPlain) pnmType = PNM_P1;
	    is_pbm = true;
	} else if (!pccasecmpASCII(cpc, "pgm")) {
	    pnmType = PNM_P5;
	    if (TxtPlain) pnmType = PNM_P2;
	    is_pgm = true;
	} else if (!pccasecmpASCII(cpc, "ppm")) {
	    pnmType = PNM_P6;
	    if (TxtPlain) pnmType = PNM_P3;
	} else if (!pccasecmpASCII(cpc, "pam")) {
	    pnmType = PNM_P7;
	    TxtPlain = false;
	    ALPHA = !(self->out_fu & fuWR_NoMask) && alpha_akt(self);
	} else {
	    ColCount = TImage_GetColorCount(self, pxgcc_RGB_ALL);
	    if (ColCount == 2) {
		pnmType = PNM_P4;
		if (TxtPlain) pnmType = PNM_P1;
		is_pbm = true;
	    } else {
		pnmType = PNM_P6;
		if (TxtPlain) pnmType = PNM_P3;
	    }
	}

	iobsize = 0;
	iobidx = 0;
	if (pnmType == PNM_P4) {
	    iobsize = ((self->width * self->height) + 7) / 8;
	    iobmem = xcalloc(iobsize);
	}

	buf = xmalloc((self->width*4)+64);
	self->tIO = (void*)buf;
	pc = buf;
	if (pnmType == PNM_P7) {
	    ppcmove(&pc, "P7\nWIDTH ");
	    inttoppc(&pc, self->width);
	    ppcmove(&pc, "\nHEIGHT ");
	    inttoppc(&pc, self->height);
	    ppcmove(&pc, "\nDEPTH ");
	    if (ALPHA) *pc = '4'; else  *pc = '3';
	    pc++;
	    ppcmove(&pc, "\nMAXVAL 255\nTUPLTYPE RGB");
	    if (ALPHA) ppcmove(&pc, "_ALPHA");
	    ppcmove(&pc, "\nENDHDR\n");
	} else {
	    *pc = 'P'; pc++;
	    *pc = pnmType | 0x30; pc++;
	    *pc = '\n'; pc++;
	    pc = wr_colval(pc, self->width);
	    inttoppc(&pc, self->height);
	    if (is_pbm) ppcmove(&pc, "\n"); else ppcmove(&pc, "\n255\n");
	}
	write_fb(self, pc);

	rgb565 = (self->pxm_LD & pxmLD_bppMask) == pxm_2b_16bpp;

  byte *sb0 = self->data;
  byte *sbA = self->rgb565_alpha_data;
	A = 255;
	for (int y=self->height;y;y--) {
	    pc = buf;
	    m = 0;
	    byte *sb = sb0;
	    for (int x=self->width;x;x--) {
  dword R, G, B;
		if (rgb565) {
		    dword c = *((word*)sb);
		    sb += 2;
		    B = (c & 0x1f)<<3;
		    G = (c & 0x7e0)>>3;
		    R = (c & 0xf800)>>8;
		    if (sbA) A = *sbA++;
		} else {
		    B = *sb++;
		    G = *sb++;
		    R = *sb++;
		    A = *sb++;
		}
		if (is_pbm) {
		    B = B + G + R;
		    if (B > 383) ch = '0'; else  ch = '1';
		    if (TxtPlain) {
			*pc = ch; pc++;
			*pc = 0;
		    } else {
			iobmem[iobidx] |= (ch & 1) << (7 - m);
			m++;
			if (m > 7) {
			    m = 0;
			    iobidx++;
			}
		    }

		} else if (is_pgm) {
		    B = (B + G + R) / 3;
		    if (TxtPlain) {
			pc = wr_colval(pc, B);
			m++;
			if (m > 15) {
			    m = 0;
			    pc[-1] = '\n';
			    write_fb(self, pc);
			    pc = buf;
			}
		    } else {
			*pc++ = B;
		    }

		} else {

		    if (TxtPlain) {
			pc = wr_colval(pc, R);
			pc = wr_colval(pc, G);
			pc = wr_colval(pc, B);
			m++;
			if (m > 5) {
			    m = 0;
			    pc[-1] = '\n';
			    write_fb(self, pc);
			    pc = buf;
			}
		    } else {
			*pc++ = R;
			*pc++ = G;
			*pc++ = B;
			if (ALPHA) *pc++ = A;
		    }
		}
	    }

	    if (pc > buf) {
		if (TxtPlain) {
		    if (pc[-1] == ' ') {
			pc[-1] = '\n';
		    } else {
			*pc = '\n'; pc++;
			*pc = 0;
		    }
		}
		write_fb(self, pc);
	    }
	    sb0 += self->bytes_per_line;
	}

	if (pnmType == PNM_P4) {
	    fwrite(iobmem, 1, iobsize, self->out_fp);
	    xfree(iobmem);
	}

	xfree(buf);
	return IMGWR_OK;
}

static bool ChkType(TglobPath *gp, const char *fn, FILE *F) {
	return TImage_ChkPattern(gp, fn, hImageWR_pnm.pattern);
}

const ThImageLoader hImageWR_pnm = {
  "pnm pbm pgm ppm pam",
  filetyp_PNM,
  ChkType,
  write_pnm
};
