/*
 Copyright (C) 2010-2022 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "appbase.h"
#include <unistd.h>
#include <sys/wait.h>
#include <errno.h>
#include <fcntl.h>

TappDat appDat;

void (*FAppRun)();
static int run_depth;
static fd_set in_fdset, out_fdset;

#ifdef TEST_NOTIFY_TIME
dword test_notify_us = 0;
static dword az_notify_us;
#endif

#if AppEvCountAz10s
dword t10sec;
dword EvCount;
dword Last_us;
bool DisableAppEvCountAz10s;
#endif




static void Ev_QUIT(int nonsense) {
  appDat.gQUIT_Ev = true;
//  appDat.gQUIT_Ev_Mk = true;
}

/* static void Ev_SIGPIPE(int nonsense) {
  appDat.gSIGPIPE_Ev = true;
  printf("ERROR %s(%d): slave_pipe Ev_SIGPIPE\n", FL);
} */

static void set_app_time() {
  uint64_t uval;
	gettimeofday(&appDat.app_tv, NULL);
	uval = ((uint64_t)appDat.app_tv.tv_sec * 1000000) + 
		(uint64_t)appDat.app_tv.tv_usec;
	appDat.app_us = uval;
	appDat.Hz30 = uval / 33333;
	appDat.app_ms = uval / 1000;
//(appDat.app_tv.tv_sec * 1000) + (appDat.app_tv.tv_usec / 1000);
}

void initapp(int fu, int argc, char **argv) {
  const int SigTab[] = {SIGHUP, SIGQUIT, SIGKILL, SIGTERM};
  int i;
  char tmpf[4096];

	app_name = GetFileName_pc(*argv);
#ifdef DEBUG_STACK
	signal(SIGSEGV, abort_SIGSEGV);
#endif

	set_app_time();
	appDat.app_start_us = appDat.app_us;
	appDat.app_start_ms = appDat.app_ms;
	appDat.app_start_sec = appDat.app_tv.tv_sec;
	appDat.gQUIT_Ev = false;
	appDat.gSIGPIPE_Ev = false;

	for (i=0; i < 4; i++) signal(SigTab[i], Ev_QUIT);
	fd_set_count(31);

	i = readlink("/proc/self/exe", tmpf, sizeof(tmpf));
	if (i>0) appDat.self_exe = pcndup(tmpf, i);

	appDat.self_pid = getpid();
	appDat.sHOME = getenv("HOME");
#ifdef APP_INI_DAT
	if (!appDat.sHOME) appDat.sHOME = "/tmp";
	appDat.ALDini_dir = pcadd(appDat.sHOME, "/.ALDccp");
	if ((fu & app_NoIni) == 0) {
//	    file_mkdir_erl(appDat.ALDini_dir, 0750, EMSG_ERR_EXIT);
	    file_mkdir_parents_ae(appDat.ALDini_dir, 0750, EMSG_ERR_EXIT);
	}
#endif
//	SetCPU_Flags(NULL);

#ifdef TEST_NOTIFY_TIME
	az_notify_us = GetTime_us();
#endif
#if AppEvCountAz10s
	t10sec = GetTime_us() + 10000000;
#endif
}


bool AppProcessMessages() {
	if (appDat.TimerDispatch) appDat.TimerDispatch();
	if (appDat.X11DispatchEvent) appDat.X11DispatchEvent();
	if (appDat.MSG_Buf_Ev) appDat.MSG_Buf_Ev();
	if (appDat.gQUIT_Ev) return false;
	if (appDat.Wins_PaintOnEvEnd) appDat.Wins_PaintOnEvEnd();
	return true;
}

bool fd_in_wait_sigle(int fd, int us) {
  int result;
  fd_set tmpfd;
  struct timeval tv;

// printf("%s(%d): fd_in_wait_sigle fd=%d us=%d\n", FL, fd, us);
	if (fd <= 0) return false;
	if (us < 0) us = 0;
	FD_ZERO (&tmpfd);
	FD_SET (fd, &tmpfd);
	set_timeval(&tv, us);
	appDat.Sleep = true;
	result = select(appDat.fd_set_count, &tmpfd, NULL, 0, &tv);
	appDat.Sleep = false;
	if (result <= 0) return false;
	return FD_ISSET(fd, &tmpfd) != 0;
}

void AppSetWait0_single() {
	atomic_or(&appDat.appFu, appFu_select_t0_single);
}
void AppSetWait1_single() {
	atomic_or(&appDat.appFu, appFu_select_t1_single);
}

static int app_t_single;
void AppSetWait_single(int us) {
	if (us > 500000) us = 500000;
	if (us < 0) us = 0;
	app_t_single = us;
	atomic_or(&appDat.appFu, appFu_select_t_single);
}

void Free_pid_Handle(int pid, int stat_loc) {
	if ((pid > 0) && appDat.Pid_Array) {
	    TPid_Array *pida, **ppida;
	    ppida = &appDat.Pid_Array;
	    while (*ppida && (((*ppida)->pid) != pid)) ppida = &(*ppida)->next;
	    pida = *ppida;
	    if (pida) {
		*ppida = pida->next;
		if (pida->On_pid_close) pida->On_pid_close(pid, stat_loc, pida);
		xfree(pida);
	    }
	}
}

bool AppHandleMessageRun() {
  int i, retval, fd_count, us, stat_loc;
  byte *pPipe;
  struct timeval tv;
  bool ret;

	run_depth++;
	if (run_depth > 100) {
	    fprintf(stderr, "%s(%d) ERROR: AppHandleMessageRun run_depth > 100\n", FL);
	    Close();
	    return false;
	}

	FD_ZERO (&in_fdset);
	FD_ZERO (&out_fdset);
	if (appDat.x_fd > 0) FD_SET (appDat.x_fd, &in_fdset);
	fd_count = appDat.fd_set_count;

	pPipe = appDat.AktPipes;
	for (i=0; i<fd_count; i++) {	//max.FD_SETSIZE
	    if (*pPipe) {
		if (*pPipe & OutPipe) {
		    if (*pPipe == OutPipe) FD_SET(i, &out_fdset); else *pPipe -= 1;
		} else {
		    if (*pPipe == InPipe) FD_SET(i, &in_fdset); else *pPipe -= 1;
		}
	    }
	    pPipe++;
	}

	appDat.Sleep = true;
	us = 500000;
	if (appDat.appFu & appFu_select_t_single) us = app_t_single;
	if (appDat.appFu & appFu_select_t1_single) us = 10000;
	if ((appDat.appFu & (appFu_select_t_NULL|appFu_select_t0_single)) ||
	    appDat.thd_private_EV) us = 0;

	appDat.tapp_us += GetTime_us() - (dword)appDat.app_us;

	appDat.Sleep = true;
	set_timeval(&tv, us);
	retval = select(appDat.fd_set_count, &in_fdset, &out_fdset, 0, &tv);
	appDat.Sleep = false;

	atomic_and(&appDat.appFu, ~(appFu_select_t0_single|appFu_select_t1_single|appFu_select_t_single));
	set_app_time();

#if AppEvCountAz10s
    if (!DisableAppEvCountAz10s) {
	EvCount++;
	dword t10s = (appDat.app_us - appDat.app_start_us) / 10000000;
	if (t10sec != t10s) {
	    t10sec = t10s;
	    Tpid_stat pid_stat;
	    char buf[80];
	    snprintf(buf, sizeof(buf), "/proc/%d/stat", appDat.self_pid);
	    if (get_pid_stat(&pid_stat, buf, PIDSTAT_NO_NAME)) {
		float proz = (float)(pid_stat.utime + pid_stat.stime) * 1000000 / (float)((appDat.app_us - appDat.app_start_us));
    printf("%s(%d): ev=%-5u u=%-4u %3.3f%% us=%-6u -> %u\n", FL, EvCount, pid_stat.utime + pid_stat.stime, proz, appDat.tapp_us, appDat.tapp_us - Last_us);
	    }
	    Last_us = appDat.tapp_us;
	}
    }
#endif

// unsigned int *pfd = &in_fdset;
	if (retval > 0) {
	  Tfd_list *fdl = appDat.fd_list;
	  for (i=0; i<fd_count; i++) {
	    if (FD_ISSET(i, &in_fdset)) {
	        if (fdl->Proc) fdl->Proc(i, fdl);
		FD_CLR(i, &in_fdset);
	    } else if (FD_ISSET(i, &out_fdset)) {
	        if (fdl->Proc) fdl->Proc(i, fdl);
		FD_CLR(i, &out_fdset);
	    }
	    fdl++;
	  }

	} else if (retval == -1) {
	  if (errno == EBADF) {  //9	Bad file number
	    pPipe = appDat.AktPipes;
	    for (i=0; i<fd_count; i++) {
	      if (*pPipe && (fcntl(i, F_GETFD) == -1)) {
		if (appDat.OnPipeError) appDat.OnPipeError(i, EBADF);
		*pPipe = 0;
		appError(EMSG_ERR1, "%s(%d) : Pipe %d Bad file number \n", FL, i);
	      }
	      pPipe++;
	    }
	  } else if (errno == EINTR) {  //4  Interrupted system call  (TimeOut)

	  } else {
 printf("%s(%d): pid=%d CLR: errno=%d\n", FL, appDat.self_pid, errno);
	  }
	  if (run_depth > 1) {
	      FD_ZERO (&in_fdset);	// fdset is not ZERO !!!
	      FD_ZERO (&out_fdset);
	  }
	}

	int pid = waitpid(-1, &stat_loc, WNOHANG);
	if (pid > 0) Free_pid_Handle(pid, stat_loc);

	if (appDat.thd_private_EV) appDat.thd_private_EV();

	ret = AppProcessMessages();

	if (appDat.appEv_Array) {
  TEv_Array *pev, **ppev;
	    ppev = &appDat.appEv_Array;
	    while (*ppev) {
		pev = *ppev;
		if (pev->run_mk) {
		    ppev = &pev->next;
		} else {
		    if (pev->On_appEv) {
			pev->run_mk = true;
			pev->On_appEv(pev);
			pev->run_mk = false;
		    }
		    if (pev->On_appEv) {
		       ppev = &pev->next;
		    } else {
			*ppev = pev->next;
			xfree(pev);
		    }
		}
	    }
	}

#ifdef MEM_CHK
#ifdef MEM_CHKappAZ
	MEM_CHK_AZ();
#endif
#endif
#ifdef TEST_NOTIFY_TIME
	if (GetCurTimeDiv_us(az_notify_us) > 10000000) {
	    printf("Debug %s(%d): notify_us=%uus\n", FL, test_notify_us);
	    az_notify_us = GetTime_us();
	    test_notify_us = 0;
	}
#endif
	run_depth--;
	return ret;
}

bool AppHandleMessage() {
	AppSetWait1_single();
	return AppHandleMessageRun();
}

extern void Exit_ClrAllTimer();

void AppRun() {
	run_depth = 0;
	appDat.AppRun = true;
	if (FAppRun) {
	    FAppRun();
	    appDat.AppExit = true;
	} else {
	    while (AppHandleMessageRun());
	    appDat.AppExit = true;
	}
	appDat.AppRun = false;
	app_atexit();
	Exit_ClrAllTimer();
}

void Close() {
	appDat.gQUIT_Ev = true;
}

#ifdef MEM_CHK
void Free_appDat() {

	ppTfree(&app_mime);
	ppcfree(&appDat.ALDini_dir);
	ppcfree(&appDat.inifile_global);
	ppcfree(&appDat.inifile_app);
	Tfree(appDat.appIni);

	while (appDat.appEv_Array) {
	    TEv_Array *p = appDat.appEv_Array;
	    appDat.appEv_Array = p->next;
	    xfree(p);
	}

	while (appDat.Pid_Array) {
	    TPid_Array *p = appDat.Pid_Array;
	    appDat.Pid_Array = p->next;
	    xfree(p);
	}
	ppcfree(&appDat.self_exe);
	appDat.fd_set_count = 0;
	ppcfree(&appDat.fd_list);
}
#endif
