/*
 Copyright (C) 2010-2022 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "sysbase.h"

static char *appstr_va(const char *msg, va_list args) {
  char buf[2048];
	vsnprintf(buf, 2047, msg, args);
	return pcdup(buf);
}

TAppMsg *AppMsgStr(const char *msg, ...) {
  va_list args;
  TAppMsgStr *msgs;
  
	if (!msg) return NULL;
	msgs = xcalloc(sizeof(TAppMsgStr));
	msgs->type = APPMSG_STR;

	va_start(args, msg);
	msgs->s = appstr_va(msg, args);
	va_end(args);

	return (TAppMsg*) msgs;
}

TAppMsg *AppMsgAppError(int ErrType, const char *msg, ...) {
  va_list args;
  TAppMsgAppError *msgs;
  
	if (!msg) return NULL;
	msgs = xcalloc(sizeof(TAppMsgAppError));
	msgs->type = APPMSG_APPERROR;
	msgs->ErrType = ErrType;

	va_start(args, msg);
	msgs->err_s = appstr_va(msg, args);
	va_end(args);

	return (TAppMsg*) msgs;
}

TAppMsg *AppMsgAppErrorBef(int ErrType, int Bef, const char *msg, ...) {
  va_list args;
  TAppMsgAppError *msgs;
  
	if (!msg) return NULL;
	msgs = xcalloc(sizeof(TAppMsgAppError));
	msgs->type = APPMSG_APPERROR;
	msgs->ErrType = ErrType;
	msgs->Bef = Bef;

	va_start(args, msg);
	msgs->err_s = appstr_va(msg, args);
	va_end(args);

	return (TAppMsg*) msgs;
}

void AppMsgFree(TAppMsg *msg) {
  TAppMsgStr *str;

	if (!msg) return;
	if (msg->ppself) *msg->ppself = NULL;
	switch (msg->type) {
	case APPMSG_STR:
	case APPMSG_ERRORSTR:
	case APPMSG_ERROR:
	case APPMSG_APPERROR:
	    str = (TAppMsgStr*)msg;
	    xfree(str->s);
	    break;
	}
	xfree(msg);
}
