/*
 Copyright (C) 2010-2022 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "appbase.h"
#include <errno.h>
#include "tthread_pv.h"

int TThread_Wait_fd(TThread *self, FileHandle fd, dword TimeOut_us) {
  int sr;
  TThread_pv *pv;
#ifdef _OS_WIN32
  HANDLE EventHandles[2];
#else
  struct timeval totv;
#endif

	pv = self->thd_pv;
	if (ChkFileHandleOK(fd)) {
#ifdef _OS_WIN32
	    TimeOut_us /= 1000;
	    EventHandles[0] = pv->Event;
	    EventHandles[1] = fd;
	    int hCount = 1;
	    if (self->fd) hCount++;
	    DWORD Signaled = WaitForMultipleObjects(hCount, EventHandles, false, TimeOut_us);
	    if (Signaled == (WAIT_OBJECT_0 + 1)) {
		return thdWait_fd;
	    }
#else
	    totv.tv_sec = TimeOut_us / 1000000;
	    totv.tv_usec = (TimeOut_us % 1000000);
	    FD_ZERO (&pv->fdset_in);
	    FD_SET(fd, &pv->fdset_in);
	    sr = select(fd+1, &pv->fdset_in, NULL, 0, &totv);
	    if (sr > 0) return thdWait_fd;

	    if (sr == 0) {
 printf("%s(%d): TThread_Wait_fd pid=%d CLR: errno=%d\n", FL, appDat.self_pid, errno);
	    }

	    if (sr < 0) {
		if (errno == EINTR) return thdWait_TimeOut;
 printf("%s(%d): TThread_Wait_fd pid=%d CLR: errno=%d\n", FL, appDat.self_pid, errno);
		 return THD_WAIT_FD_ERROR;
	    }
#endif
	}
	return thdWait_TimeOut;
}
