/*
 Copyright (C) 2010-2024 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "sysbase.h"

void *ttc_create(int tcsize, int wtyp) {
  TTC0 *tc;
	if (tcsize < sizeof(TTC0)) {
	    appError(EMSG_ERR1, "%s(%d): ttc_create: tcsize=%d < sizeof(TTC)=%d", FL, tcsize, sizeof(TTC));
	    return NULL;
	}
	tc = xcalloc(tcsize);
	tc->wtcsize = tcsize;
	if (tcsize > TTC_MAXSIZE) {
	    appError(EMSG_WARN, "%s(%d): ttc_create: wtyp=0x%x tcsize=%d > %d", FL, tcsize, TTC_MAXSIZE, wtyp);
	}
	tc->wtyp = wtyp;
	tc->magic = TTC_MAGIC;  //0xfffda559
	return tc;
}

static bool chk_struct(void *tc) {
	if (((TTC*)tc)->magic != TTC_MAGIC) {
	    appError(EMSG_ERR1, "%s(%d): 0x%lx ttc_chk_struct: not a TTC struct ???", FL, (long int)tc);
	    return false;
	}
	if (((TTC*)tc)->wtcsize < sizeof(TTC0)) {
	    appError(EMSG_ERR1, "%s(%d): 0x%lx ttc_chk_struct: tcsize=%d < sizeof(TTC)=%d", FL, (long int)tc, ((TTC*)tc)->wtcsize, sizeof(TTC));
	    return false;
	}
	return true;
}

bool ttc_chk_struct(void *tc) {
	if (!tc) return false;
	return chk_struct(tc);
}

bool REGP2 ttc_chk_is_wtyp(void *tc, int wtyp) {
	return  tc && 
		(((TTC*)tc)->magic == TTC_MAGIC) &&
		(((TTC*)tc)->wtcsize >= sizeof(TTC0)) &&
		((TTC*)tc)->wtyp == wtyp;
}

static int tc_err_count = 0;

void Tfree(void *tc) {
	if (!tc) return;
	if (!chk_struct(tc)) {
	    tc_err_count++;
	    if (tc_err_count > 10) appError(EMSG_ERR_EXIT, "%s(%d): err_count > 10", FL);
	    return;
	}
	tc_err_count = 0;
	if (((TTC*)tc)->FREE_LOCK) {
	    ((TTC*)tc)->FREE_LOCK_Stat |= TTC_GoFree;
	    return;
	}
	if (((TTC*)tc)->destroy && ((TTC*)tc)->destroy(tc)) return;
	((TTC*)tc)->magic = 0;	//set old mem free
	xfree(tc);
}

void pp_Tfree(TTC0 **pptc) {
  void *tc;
	if (!pptc) return;
	tc = *pptc;
	*pptc = NULL;
	Tfree(tc);
}

void TfreeLock(void *tc) {
	if (!ttc_chk_struct(tc)) return;
	((TTC*)tc)->FREE_LOCK++;
}

bool TfreeUnLock(void *tc) {
  bool result;
	if (!ttc_chk_struct(tc)) return true;
	result = (((TTC*)tc)->FREE_LOCK_Stat & TTC_GoFree) != 0;
	if (((TTC*)tc)->FREE_LOCK) ((TTC*)tc)->FREE_LOCK--;
	if (!((TTC*)tc)->FREE_LOCK && result) Tfree(tc);
	return result;
}
