/*
 Copyright (C) 2010-2024 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

#include "aldesk.h"
#include <unistd.h>
#include <fcntl.h>
#include <scsi/sg.h>
#include <sys/ioctl.h>
#include <scsi/scsi.h>
//#include <scsi/scsi_ioctl.h>

#ifdef	USE_DEBUG_WIN
int GrabXServer_count = 0;
#else
static int GrabXServer_count = 0;
#endif

bool chk_dev_removable(char *fn) {
#if 1
  int fd, status, i;
  sg_io_hdr_t io_hdr;
  unsigned char sense_buffer[32];
  unsigned char inqBuff[2];
  unsigned char cmdp[6];

	fd = open(fn, O_RDONLY | O_NONBLOCK);
	if (fd > 0) {
	    status = ioctl(fd, SG_GET_VERSION_NUM, &i);
	    if ((status < 0) || (i < 30000)) {
//		fprintf(stderr, "ERROR: not a sg device or old sg driver\n");
		return false;
	    }
	    Fill0(&io_hdr, sizeof(io_hdr));
	    Fill0(&cmdp, sizeof(cmdp));
	    io_hdr.interface_id = 'S';
	    io_hdr.cmd_len = 6;
	    io_hdr.mx_sb_len = sizeof(sense_buffer);
	    io_hdr.dxfer_direction = SG_DXFER_NONE;
	    /* io_hdr.dxfer_len = 0; */
	    io_hdr.dxferp = inqBuff;
	    io_hdr.cmdp = cmdp;
	    io_hdr.sbp = sense_buffer;
	    io_hdr.timeout = 2000;

	    cmdp[0] = ALLOW_MEDIUM_REMOVAL;
	    status = ioctl(fd, SG_IO, (void *)&io_hdr);
	    close(fd);

	    if ((status >= 0) && 
	        (io_hdr.host_status == 0) && 
		(io_hdr.driver_status == 0)) return true;
	}
	return false;

#else
  int fd, status;
  struct sdata {
	int  inlen;
	int  outlen;
	char cmd[256];
  } scsi_cmd;

	fd = open(fn, O_RDONLY | O_NONBLOCK);
	if (fd > 0) {
	    Fill0(&scsi_cmd, sizeof(scsi_cmd));
	    scsi_cmd.cmd[0] = ALLOW_MEDIUM_REMOVAL;
	    status = ioctl(fd, SCSI_IOCTL_SEND_COMMAND, (void *)&scsi_cmd);
	    close(fd);
	    if (!status) return true;
	}
	return false;
#endif
}

void SetXInputFocus(Window WinID) {
	XSetInputFocus (appXDat.dpy, WinID, RevertToParent, 0);
}

void GrabXServer() {
	if (!GrabXServer_count) XGrabServer(appXDat.dpy);
	GrabXServer_count++;
}

void UngrabXServer() {
	if (!GrabXServer_count) return;
	GrabXServer_count--;
	if (!GrabXServer_count) XUngrabServer(appXDat.dpy);
}

void SyncX() {
	XSync(appXDat.dpy, 0);
}

int get_bef_dflEv(TFileMsg *fmsg, int *r_Bef) {
  int bef;

	if (r_Bef) *r_Bef = 0;
	if (!file_msg_chk_wtyp(fmsg, wtyp_TDirListObjNy, FL)) return 0;

	if (fmsg->art != fmArt_DirList) return 0;
	if (fmsg->typD != fmTyp_int) return 0;
	if (r_Bef) *r_Bef = fmsg->D.m;
	if (fmsg->typF != fmTyp_dfl) return 0;
	if (fmsg->typR != fmTyp_gp) return 0;
	bef = fmsg->D.m & dlbef_BefMask;

	if (fmsg->D.m & dlbef_ScanRun) {
	    if (bef == dlbef_Add) return dlbef_Add;
	} else if (fmsg->D.m & dlbef_NotifyRun) {
	    if ((bef == dlbef_Add) || (bef == dlbef_Change)) return 0;
	    return bef;
	}
	return 0;
}
