/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "appXbase.h"
#include "thexedit.h"
#include <fcntl.h>

bool THexEdit_LoadFile(THexEdit *self, const char *FileName) {
  TglobPath *gp;
  Tstat64 *st;
  const char *pcerr = NULL;
  bool result;

	ppTfree(&self->fio);

	gp = pctogp(FileName);
	st = file_stat_gp(gp, fsFollowLink | fsGetFileType | fsNoFileName | fsNoLinkName);
	if (!st || (st == PTR_FAILED)) {
	    if (!st) pcerr = "file not found";
	    st = NULL;

#ifdef _OS_WIN32
	} else if (!(st->st & st_Reg)) {
#else
	} else if (!(st->st & st_Reg) && (((st->mode & S_IFMT) != S_IFBLK) &&
		(((st->mode & S_IFMT) != S_IFCHR) || !pccmp("/dev/mem", FileName)))) {
#endif
	    pcerr = "file is not a Regular file !!!";
	}

	if (pcerr) {
	    char *fn = get_DisplayFileStr(NULL, gp);
	    TTC_netError((TTC0*)self, NULL, EMSG_ERR1, "%s \n%s", pcerr, fn);
	    xfree(fn);

	} else if (st) {
#ifdef _OS_WIN32
	    self->fio = TFIO_Create_LB(FileName);
#else
	    if (((st->mode & S_IFMT) == S_IFBLK) || 
		((st->mode & S_IFMT) == S_IFCHR) ||
		(st->size >= 0x00400000)) { 
		self->fio = TFIO_Create_FD(FileName);
	    } else {
		self->fio = TFIO_Create_LB(FileName);
	    }
#endif
	    THexEdit_fioSetUp(self);
	}

	xfree(st);
	gp_free(gp);

	result = FIO_Open(self->fio);
	THexEdit_Adjust_VHSB(self);
	Refresh((TWins*)self);
	return result;
}
