/*
 Copyright (C) 2010-2017 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/


#ifdef FILTER_PULS

#include "ElMath.h"
#include <math.h>

void TFormPuls_SetTrigV(TFormPuls *self, float U, int fu) {
  int nk;

	self->trg_flag = 0;
	self->trg_V1 = U * self->trg_V1k;
	self->trg_V2 = U * self->trg_V2k;

	nk = 3;
	if (U >= 10.0) nk = 2;
	if (U >= 100.0) nk = 1;
	TDVal_Set_nk(self->dvOutAmpl, nk);
	TDVal_Set_Value(self->dvOutAmpl, U);
}

void TFormPuls_ChkTrigV(TFormPuls *self, float U, int idx) {
  float ns;

	if (!(self->trg_flag & TRG_LH_V1)) {
	    if (U >= self->trg_V1) {
		self->trg_flag |= TRG_LH_V1;
		self->trg_V1_ns = self->ns;
	    }

	} else if (!(self->trg_flag & TRG_LH_V2)) {
	    if (U >= self->trg_V2) {
		self->trg_flag |= TRG_LH_V2;
		self->trg_V2_ns = self->ns;
		ns = self->trg_V2_ns - self->trg_V1_ns;
		TDVal_Set_Value(self->dvTrgNs, ns);
		TDVal_Set_Value(self->dvTrgVus, (self->trg_V2 - self->trg_V1) * 1000 / ns);
/*
 printf("%6.2f %6.2f %6.2fns  %6.1fV/us\n", self->trg_V1_ns, self->trg_V2_ns, 
	self->trg_V2_ns - self->trg_V1_ns, 
	(self->trg_V2 - self->trg_V1) * 1000 / (self->trg_V2_ns - self->trg_V1_ns));
*/
	    }

	}
}

static float getVppAdd(TFormPuls *self, float ampl) {
  float Vpp_add, ns;

	ns = self->dvNs->dval;
	if (self->cbVus->Checked) {
	    ns = ampl * 1000 / self->dvVus->dval;
	}
	Vpp_add = ampl * self->ns_add / ns;
	if (Vpp_add > ampl) Vpp_add = ampl;
	return Vpp_add;
}

float TFormPuls_get_ps_add(TFormPuls *self, int count, int k) {
  float ml;

	
	if (k < 1) k = 1;
	ml = count * k;
	self->ns_add = self->T->dval * 10 / ml;
	self->nsPp = self->ns_add * ml * self->dvPp->dval / 100;
	self->Vpp_add = getVppAdd(self, self->dvVpp->dval);
// printf("%f %f %f\n", self->ns_add, self->nsPp, self->Vpp_add);
	self->ns = 0;
	self->ampl = 0;
	self->istgl = false;
	return self->ns_add * 1000;
}

float TFormPuls_get_ampl(TFormPuls *self) {
	if (self->cbPp->Checked && (self->ns >= self->nsPp)) {
	    if (!self->istgl) {
		self->istgl = true;
		self->Vpp_add = getVppAdd(self, self->ampl);
	    }
	    if (self->ampl > 0.0) self->ampl -= self->Vpp_add;
	    if (self->ampl < 0.0) self->ampl = 0.0;
	} else {
	    if (self->ampl < self->dvVpp->dval) self->ampl += self->Vpp_add;
	    if (self->ampl > self->dvVpp->dval) self->ampl = self->dvVpp->dval;
	}
	self->ns += self->ns_add;
	return self->ampl;
}

void TFormPuls_SaveData(TFormPuls *self) {
  TpcIni *ini;
  int OnOff;
  char *pc;
  char buf[500];

	ini = get_appIni();
	OnOff = 0;
	if (self->cbVus->Checked) OnOff |= 0x01;
	if (self->cbPp->Checked) OnOff |= 0x02;
	pc = buf;
	hexto0xppc(&pc, OnOff, 4 | kSPACE_ADD);
	doubletoppc(&pc, self->T->dval, 1 | kSPACE_ADD);
	doubletoppc(&pc, self->Vdiv->dval, 2 | kSPACE_ADD);
	doubletoppc(&pc, self->dvVpp->dval, 2 | kSPACE_ADD);
	doubletoppc(&pc, self->dvVus->dval, 1 | kSPACE_ADD);
	doubletoppc(&pc, self->dvNs->dval, 2 | kSPACE_ADD);
	doubletoppc(&pc, self->dvPp->dval, 0 | kSPACE_ADD);

	TpcIni_WR(ini, self->Gruppe, self->sOpt, buf);
	TpcIni_Save(ini);
	free_appIni(ini);
}

void TFormPuls_SavePosOnOff(TFormPuls *self) {
  TpcIni *pcIni;
  char buf[256];

	pcIni = get_appIni();
	snprintf(buf, sizeof(buf), "%d %d %ld", self->left, self->top, self->tag);
	TpcIni_WR(pcIni, self->Gruppe, self->sXY, buf);
	TpcIni_Save(pcIni);
	free_appIni(pcIni);
}

/* ------------------------- ev ----------------------------------------- */
static void do_OnChange(TWins *any) {
  TFormPuls *form = (TFormPuls*)any->FormStruct;


	TFormPuls_SaveData(form);
	if (form->OnChange) form->OnChange(form);
}

static void cbVus_OnClick(TChkBox *self, long KeyState, bool DblClick) {
  TFormPuls *form = (TFormPuls*)self->FormStruct;

	if (self->Checked) return;
	TChkBox_SetChecked(form->cbVus, self == form->cbVus);
	TChkBox_SetChecked(form->cbns,  self == form->cbns);
	do_OnChange((TWins*)self);
}

static void cbAny_OnClick(TChkBox *self, long KeyState, bool DblClick) {
	do_OnChange((TWins*)self);
}

static void dvAny_OnValChange(TDVal *self, double val) {
	do_OnChange((TWins*)self);
}

static void dvVpp_OnValChange(TDVal *self, double val) {
  TFormPuls *form = (TFormPuls*)self->FormStruct;

	if (form->cbVus->Checked) {
	    TDVal_Set_Value(form->dvNs, val * 1000 / form->dvVus->dval);
	} else {
	    TDVal_Set_Value(form->dvVus, val * 1000 / form->dvNs->dval);
	}
	do_OnChange((TWins*)self);
}



static void dvVus_OnValChange(TDVal *self, double val) {
  TFormPuls *form = (TFormPuls*)self->FormStruct;

	TDVal_Set_Value(form->dvNs, form->dvVpp->dval * 1000 / val);
	do_OnChange((TWins*)self);
}

static void dvNs_OnValChange(TDVal *self, double val) {
  TFormPuls *form = (TFormPuls*)self->FormStruct;

	TDVal_Set_Value(form->dvVus, form->dvVpp->dval * 1000 / val);
	do_OnChange((TWins*)self);
}

static void cobTrg_OnClick(TMenu *self, long KeyState, TMenuItem *mi) {
  const char *cpc;
  TFormPuls *form;

	cpc = mi->s;
	form = (TFormPuls*)self->FormStruct;
	form->trg_V1k = ppctofloat(&cpc) / 100;
	cpc++;
	form->trg_V2k = ppctofloat(&cpc) / 100;
	
//	do_OnChange((TWins*)self);
	if (form->OnChange) form->OnChange(form);
}

static void PaintStr(TWins *self, int x, int y, const char *s) {
  int push_lg;
  char *push_s;

	if (!s || !*s) return;

	push_lg = self->vds.lg;
	push_s = self->vds.s;

	self->vds.lg = pclen(s);
	self->vds.s_const = s;

	DrawColorString(x, y, &self->vds);

	self->vds.s = push_s;
	self->vds.lg = push_lg;
}

static void PaintStrRel(TWins *self, TWins *obj, int x, int y, const char *s) {
  int xr;
	if (x < 0) {
	    xr = obj->left + x;
	    if (xr < 1) xr = 1;
	} else {
	    xr = obj->left + obj->width + x;
	}
	PaintStr(self, xr, obj->top + y, s);
}

static void pnlOpt_OnPaint(TWins *self) {
  TFormPuls *form;

	form = (TFormPuls*)self->FormStruct;
	PaintStrRel(self, (TWins*)form->dvVpp,  1, 2, "Vpp");
	PaintStrRel(self, (TWins*)form->dvVus, 1, 2, "V/us");
	PaintStrRel(self, (TWins*)form->dvNs,  1, 2, "ns");
	PaintStrRel(self, (TWins*)form->dvPp,  1, 2, "%");
}

static void pnlT_OnPaint(TWins *self) {
  TFormPuls *form;

	form = (TFormPuls*)self->FormStruct;
	PaintStrRel(self, (TWins*)form->T,  1, 3, "ns/Dev");
	PaintStrRel(self, (TWins*)form->Vdiv,  1, 2, "V/Dev");
}

static void pnlZ_OnPaint(TWins *self) {
  TFormPuls *form;

	form = (TFormPuls*)self->FormStruct;
	PaintStrRel(self, (TWins*)form->dvOutAmpl,  1, 2, "Vpp");
	PaintStrRel(self, (TWins*)form->cobTrg,  -999, 2, "Trig.");
	PaintStrRel(self, (TWins*)form->dvTrgNs,  1, 2, "ns");
	PaintStrRel(self, (TWins*)form->dvTrgVus,  1, 2, "V/us");
}

static void Form_OnMove(TFormPuls *self) {
	TFormPuls_SavePosOnOff(self);
}

TFormPuls *Create_Form_Puls(TWins *parent, const char *Caption, 
			const char *Gruppe, const char *sC) {
  TFormPuls *self;
  TpcIni *ini;
  int OnOff, y;
  const char *sopt;

	ini = get_appIni();

	self = TForm_CreatParamAlg(parent, sizeof(TFormPuls));
	self->wtyp = wtyp_TFormPuls;
	self->FormZStyle = fszTransient;
	self->FormPosition = poDesigned;
	self->Gruppe = Gruppe;
	self->sOpt = pcadd(sC, "opt");
	self->sXY = pcadd(sC, "xy");

	SetVisible((TWins*)self, false);
	SetCaption((TWins*)self, Caption);
	self->width = 236;
	self->height = 148;
	self->min_width = self->width;
	self->min_height = self->height;
	self->max_width = self->width;
	self->max_height = self->height;
	sopt = TpcIni_RD_pc(ini, Gruppe, self->sXY, "0 0");
	self->left = ppctoint(&sopt);
	self->top = ppctoint(&sopt);
	self->tag = ppctoint(&sopt);
	self->OnMove = Form_OnMove;


	self->pnl1 = TWins_CreateBA((TWins*)self, 0, 0, 134, 21, 0, alLeft);
	self->pnlT = TWins_CreateBA(self->pnl1, 0, 0, 134, 72, mBorder | mBorderL, alTop);
	self->pnlOpt = TWins_CreateBA(self->pnl1, 0, 20, 130, 80, mBorder | mBorderL, alClient);
	self->pnlZ = TWins_CreateBA((TWins*)self, 136, 0, 130, 80, mBorder | mBorderL, alClient);

	sopt = TpcIni_RD_pc(ini, self->Gruppe, self->sOpt, "0x00 1 0.1 0.5 500 1 50");
	OnOff = ppctoint(&sopt);

/* Panel T */
	self->pnlT->OnPaint = pnlT_OnPaint;
	self->T = TDVal_CreateS(self->pnlT, 16, 2, 70, 21);
	self->T->nk = 1;
	TDVal_Set_Min(self->T, 0.1);
	TDVal_Set_Value(self->T, ppctodouble(&sopt));
	self->T->dv_fu = tdv_fu_LostFocusEnter;
	self->T->OnValChange = dvAny_OnValChange;
	SetFont((TWins*)self->T, "-*-helvetica-bold-r-*-*-14-*-75-75-*-*-*-*");

	self->Vdiv = TDVal_CreateS(self->pnlT, 26, 25, 60, 18);
	self->Vdiv->nk = 2;
	TDVal_Set_Min(self->Vdiv, 0.01);
	TDVal_Set_Value(self->Vdiv, ppctodouble(&sopt));
	self->Vdiv->dv_fu = tdv_fu_LostFocusEnter;
	self->Vdiv->OnValChange = dvAny_OnValChange;

/* Panel Optionen */
	self->pnlOpt->OnPaint = pnlOpt_OnPaint;

	y = 1;
	self->dvVpp = TDVal_CreateS(self->pnlOpt, 26, y, 60, 18);
	self->dvVpp->nk = 2;
	self->dvVpp->dv_fu = tdv_fu_LostFocusEnter;
	TDVal_Set_Min(self->T, 0.01);
	TDVal_Set_Value(self->dvVpp, ppctodouble(&sopt));
	self->dvVpp->OnValChange = dvVpp_OnValChange;


	y += 19;
	self->cbVus = TChkBox_CreateS(self->pnlOpt, 2, y+2, 16, 16, NULL);
	self->cbVus->AutoToggle = false;
	TChkBox_SetChecked(self->cbVus, (OnOff & 0x01) != 0);
	self->cbVus->OnClick = cbVus_OnClick;
	self->dvVus = TDVal_CreateS(self->pnlOpt, 26, y, 60, 18);
	self->dvVus->nk = 1;
	self->dvVus->dv_fu = tdv_fu_LostFocusEnter;
	TDVal_Set_Value(self->dvVus, ppctodouble(&sopt));
	self->dvVus->OnValChange = dvVus_OnValChange;



	y += 19;
	self->cbns = TChkBox_CreateS(self->pnlOpt, 2, y+2, 16, 16, NULL);
	self->cbns->AutoToggle = false;
	TChkBox_SetChecked(self->cbns,  (OnOff & 0x01) == 0);
	self->cbns->OnClick = cbVus_OnClick;
	self->dvNs = TDVal_CreateS(self->pnlOpt, 26, y, 60, 18);
	self->dvNs->nk = 2;
	self->dvNs->dv_fu = tdv_fu_LostFocusEnter;
	TDVal_Set_Min(self->dvNs, 0.01);
	TDVal_Set_Value(self->dvNs, ppctodouble(&sopt));
	self->dvNs->OnValChange = dvNs_OnValChange;

	y += 19;
	self->cbPp = TChkBox_CreateS(self->pnlOpt, 2, y+2, 16, 16, NULL);
	TChkBox_SetChecked(self->cbPp,  (OnOff & 0x02) != 0);
	self->cbPp->OnClick = cbAny_OnClick;
	self->dvPp = TDVal_CreateS(self->pnlOpt, 26, y, 60, 18);
	self->dvPp->nk = 0;
	self->dvPp->dv_fu = tdv_fu_LostFocusEnter;
	TDVal_Set_Min(self->dvPp, 1);
	TDVal_Set_Max(self->dvPp, 100);
	TDVal_Set_Value(self->dvPp, ppctodouble(&sopt));
	self->dvPp->OnValChange = dvAny_OnValChange;



	self->sbT = TScrollBarVert_CreateP(self->pnlOpt);
	self->sbT->Align = alRight;
//	self->sbT->OnCount = sbT_OnCount;
	TScrollBar_SetMaxCount(self->sbT, 100);


/* Panel Z */
	self->pnlZ->OnPaint = pnlZ_OnPaint;
	y = 1;
	self->dvOutAmpl = TDVal_CreateS(self->pnlZ, 2, y, 60, 18);
	self->dvOutAmpl->nk = 3;
	self->dvOutAmpl->ReadOnly = true;
	SetColor((TWins*)self->dvOutAmpl, 0xe8e8e8);

	y += 21;
	self->cobTrg = TComboBox_CreateS(self->pnlZ, 28, y, 72, 18, "10-90%");
	self->cobTrg->ReadOnly = true;
	TComboBox_AddItem(self->cobTrg, "10-90%", 0, cobTrg_OnClick);
	TComboBox_AddItem(self->cobTrg, "20-80%", 0, cobTrg_OnClick);
	TComboBox_AddItem(self->cobTrg, "25-75%", 0, cobTrg_OnClick);
	self->trg_V1k = 0.1;
	self->trg_V2k = 0.9;

	y += 21;
	self->dvTrgNs = TDVal_CreateS(self->pnlZ, 2, y, 60, 18);
	self->dvTrgNs->nk = 2;
	self->dvTrgNs->ReadOnly = true;
	SetColor((TWins*)self->dvTrgNs, 0xe8e8e8);

	y += 19;
	self->dvTrgVus = TDVal_CreateS(self->pnlZ, 2, y, 60, 18);
	self->dvTrgVus->nk = 1;
	self->dvTrgVus->ReadOnly = true;
	SetColor((TWins*)self->dvTrgVus, 0xe8e8e8);

	y += 19;
	self->cbAzIn = TChkBox_CreateS(self->pnlZ, 2, y, 36, 16, "+C1");
	TChkBox_SetChecked(self->cbAzIn,  (OnOff & 0x04) != 0);
	self->cbAzIn->OnClick = cbAny_OnClick;

	free_appIni(ini);

	return self;
}


#endif /* FILTER_PULS */
