/*
 Copyright (C) 2010-2017 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

#include "ElMath.h"
#include "ElGraf.h"
#include <math.h>

static void str_toGraf(TWins *Wins, const char *s, int x, int y) {
	vds_SetStr(&Wins->vds, s);
	DrawColorString(x, y, &Wins->vds);
}

static void DrawSch(Window WinID, int x, int y, int fu) {
  int xa;
  int x2 = x;
  int y2 = y+30;

	Draw_xP1_Rg(WinID, &x2, &xa, y, y2, fu);

	if ((fu & 0x0300) == 0x000) {
	    DrawTP1_Cv(WinID, &x2, 0, y, get_Pic_cC2());
	}

	if ((fu & 0x0300) == 0x100) {
	    DrawTP1_Cv(WinID, &x2, 0, y, get_Pic_cC1());
	    DrawTP1_Lh(WinID, &x2, y, get_Pic_cL1());
	    DrawTP1_Cv(WinID, &x2, 0, y, get_Pic_cC4());
	}
	if ((fu & 0x0300) == 0x200) {
	    DrawTP1_Cv(WinID, &x2, 0, y, get_Pic_cC1());
	    DrawTP1_Lh(WinID, &x2, y, get_Pic_cL1());
	    DrawTP1_Cv(WinID, &x2, 0, y, get_Pic_cC2());
	    DrawTP1_Lh(WinID, &x2, y, get_Pic_cL3());
	    DrawTP1_Cv(WinID, &x2, 0, y, get_Pic_cC4());
	}

	if ((fu & 0x0300) == 0x300) {
	    DrawTP1_Cv(WinID, &x2, 0, y, get_Pic_cC1());
	    DrawTP1_Lh(WinID, &x2, y, get_Pic_cL1());
	    DrawTP1_Cv(WinID, &x2, 0, y, get_Pic_cC2());
	    DrawTP1_Lh(WinID, &x2, y, get_Pic_cL2());
	    DrawTP1_Cv(WinID, &x2, 0, y, get_Pic_cC3());
	    DrawTP1_Lh(WinID, &x2, y, get_Pic_cL3());
	    DrawTP1_Cv(WinID, &x2, 0, y, get_Pic_cC4());
	}

	Draw_xP1_Ra(WinID, x2, xa, y, y2, fu);
}

static float ImpF1(TPage_TP1 *self, float MHz, float *Z) {
  float res;
  TRLC btIn;
  float w = MHz * M_PI * 2;
	if (!XF_MathOut((TPage_XFilter*)self, w, &btIn, &res)) return res;
	Form_C_math(self->f_C2, w);
	RLC_Add_Par(&btIn, &self->f_C2->rlc);
	res *= btIn.Z;

	XF_MathIn((TPage_XFilter*)self, w, &btIn, &res, Z);
	return res;
} 
static float ImpF2(TPage_TP1 *self, float MHz, float *Z) {
  float res;
  TRLC btIn;
  float w = MHz * M_PI * 2;
	if (!XF_MathOut((TPage_XFilter*)self, w, &btIn, &res)) return res;
	Form_C_math(self->f_C1, w);
	Form_L_math(self->f_L1, w);
	Form_C_math(self->f_C4, w);
	RLC_Add_Par(&btIn, &self->f_C4->rlc);
	res *= btIn.Z;
	RLC_Add_Ser(&btIn, &self->f_L1->rlc);
	res /= btIn.Z;
	RLC_Add_Par(&btIn, &self->f_C1->rlc);
	res *= btIn.Z;

	XF_MathIn((TPage_XFilter*)self, w, &btIn, &res, Z);
	return res;
}
static float ImpF3(TPage_TP1 *self, float MHz, float *Z) {
  float res;
  TRLC btIn;
  float w = MHz * M_PI * 2;
	if (!XF_MathOut((TPage_XFilter*)self, w, &btIn, &res)) return res;
	Form_C_math(self->f_C1, w);
	Form_C_math(self->f_C2, w);
	Form_L_math(self->f_L1, w);
	Form_L_math(self->f_L3, w);
	Form_C_math(self->f_C4, w);
	RLC_Add_Par(&btIn, &self->f_C4->rlc);
	res *= btIn.Z;
	RLC_Add_Ser(&btIn, &self->f_L3->rlc);
	res /= btIn.Z;
	RLC_Add_Par(&btIn, &self->f_C2->rlc);
	res *= btIn.Z;
	RLC_Add_Ser(&btIn, &self->f_L1->rlc);
	res /= btIn.Z;
	RLC_Add_Par(&btIn, &self->f_C1->rlc);
	res *= btIn.Z;

	XF_MathIn((TPage_XFilter*)self, w, &btIn, &res, Z);
	return res;
}
static float ImpF4(TPage_TP1 *self, float MHz, float *Z) {
  float res;
  TRLC btIn;
  float w = MHz * M_PI * 2;
	if (!XF_MathOut((TPage_XFilter*)self, w, &btIn, &res)) return res;
	Form_C_math(self->f_C1, w);
	Form_C_math(self->f_C2, w);
	Form_C_math(self->f_C3, w);
	Form_L_math(self->f_L1, w);
	Form_L_math(self->f_L2, w);
	Form_L_math(self->f_L3, w);
	Form_C_math(self->f_C4, w);
	RLC_Add_Par(&btIn, &self->f_C4->rlc);
	res *= btIn.Z;
	RLC_Add_Ser(&btIn, &self->f_L3->rlc);
	res /= btIn.Z;
	RLC_Add_Par(&btIn, &self->f_C3->rlc);
	res *= btIn.Z;
	RLC_Add_Ser(&btIn, &self->f_L2->rlc);
	res /= btIn.Z;
	RLC_Add_Par(&btIn, &self->f_C2->rlc);
	res *= btIn.Z;
	RLC_Add_Ser(&btIn, &self->f_L1->rlc);
	res /= btIn.Z;
	RLC_Add_Par(&btIn, &self->f_C1->rlc);
	res *= btIn.Z;

	XF_MathIn((TPage_XFilter*)self, w, &btIn, &res, Z);
	return res;
}

#ifdef FILTER_PULS
static int XXXXX;

static float PulsF2(TPage_TP1 *self, float ug, float ps_add) {
  float res;
#if 1
  double U, I, R, R2, R_C1, R_L1, R_L1_Rs, R_L1_Rp, R_L1_Cp, UZ, Z;
  double I_C1, I_L1, U_L1, U_L1_Load, R_L1_Load, I_L1_Load;

  XXXXX++;


	R_C1 = ps_add / self->f_C1->TC->dval;
	R_L1 = self->f_L1->TL->dval * 1000000 / ps_add;
	
	
	R_L1_Cp = ps_add / self->f_L1->Cp->dval;

 if (XXXXX == 1) printf("%f %f\n", R_L1_Cp, R_L1);

	U = ug - self->f_C1->Uc;
	I_C1 = U / R_C1;
	self->f_C1->Uc += U;


	U_L1_Load = ug; // - self->f_L1->Uc;
	R_L1_Load = self->f_Rg->TR->dval;

#if 1
	Form_L_Puls_math(self->f_L1, R_L1_Load, U_L1_Load);
	res = self->f_L1->Puls_U;
#else
// printf("%f", I_L1_Load);

	UZ = self->f_L1->Uc;

	U_L1 = 0;
	R_L1_Rp = 100000000;
	R_L1_Rs = 0.001;
	if (self->f_L1->cbOptOn->Checked) {
	    if (self->f_L1->cbRp->Checked) R_L1_Rp = self->f_L1->Rp->dval * 1000;
	    if (self->f_L1->cbRs->Checked) R_L1_Rs = self->f_L1->Rs->dval;

	    if (self->f_L1->cbCp->Checked) {
		I_L1_Load = (U_L1_Load - self->f_L1->Uc) / R_L1_Load;
		I = self->f_L1->Il + (self->f_L1->Uc / R_L1_Rp) - I_L1_Load;
		U_L1 = self->f_L1->Uc - (I * R_L1_Cp);
		self->f_L1->Uc = U_L1;
		I_L1 = (U_L1 - (self->f_L1->Il * R_L1_Rs)) / (R_L1 + R_L1_Rs);
		self->f_L1->Il += I_L1;

	    } else {
		U = U_L1_Load / ((R_L1_Load / R_L1_Rp) + 1);
		R2 = 1 / ((1 / R_L1_Load) + (1 / R_L1_Rp));
		R = R2 + R_L1_Rs;
		self->f_L1->Il += (U - (self->f_L1->Il * R)) / (R_L1 + R);
		U_L1 = U - (R2 * self->f_L1->Il);
		self->f_L1->Uc = U_L1;
	    }

	} else {
	    self->f_L1->Il += (U_L1_Load - (self->f_L1->Il * R_L1_Load)) / (R_L1 + R_L1_Load);
	    U_L1 = U_L1_Load - (R_L1_Load * self->f_L1->Il);
	    self->f_L1->Uc = U_L1;
	}

	U = self->f_L1->Uc - UZ;
	if ((U > -0.0000001) && (U < 0.0000001)) {
	    Z = 0;
	} else {
	   Z = R_L1_Load / ((U_L1_Load / U) - 1); 
	}


 printf(" %9.6f %9.6f %9.6f\n", self->f_L1->Il, self->f_L1->Uc, Z);


//	if (ug > 0.01) {
//	   U_L1 = ug;
//	} else {
//	   I = self->f_L1->Il + (self->f_L1->Uc / R_L1_Rp);
//	   U_L1 = self->f_L1->Uc - (I * R_L1_Cp);
//	}

//	U = U_L1 - self->f_L1->Uc;
//	I_L1_C1 = U / R_L1_Cp;


//	Z = U_L1 / (self->f_L1->Il + I_L1_C1);

// if (XXXXX < 10) 
// printf("%9.6f %9.6f %9.6f %9.6f uC=%9.6f %9.6f\n", I_L1_C1, I_L1, self->f_L1->Il, I_L1_C1, U_L1, Z);


	res = U_L1;
#endif

#if 0

//	Rg = self->f_Rg->TR->dval;
	R_C1 = ps_add / self->f_C1->TC->dval;
	R_C4 = ps_add / self->f_C4->TC->dval;

	Ug_L1 = ug;
	R_L1 = self->f_L1->TL->dval * 1000000 / ps_add;
	R_L1_Cp = 100000000;
//	I = 0;
	if (self->f_L1->cbOptOn->Checked && self->f_L1->cbCp->Checked) {
	    R_L1_Cp = ps_add / self->f_L1->Cp->dval;
//	    I = ((self->f_C1->Uges - self->f_C4->Uges) - self->f_L1->Uc) / R_L1_Cp;
//	    self->f_L1->Uc += I * R_L1_Cp;
	}
 if (XXXXX == 1) printf("%f %f %f %f\n", R_C1, R_L1, R_L1_Cp, R_C4);

	U = Ug_L1 - self->f_L1->Uc;
	I_L1_Cp = U / R_L1_Cp;
	self->f_L1->Uc += U * 0.63;

	I_L1 = Ug_L1 / R_L1;

	Z_L1 = Ug_L1 / (I_L1_Cp + self->f_L1->Il + I_L1);

	self->f_L1->Il += I_L1;

 printf("%f %f -> %f %f -> %f\n", I_L1, I_L1_Cp, self->f_L1->Uc, self->f_L1->Il, Z_L1);


#endif

#if 0
  double U, I, R, Rg, R_C1, R_L1, R_L1_Cp, R_C4, I_RG;

/*	R_L1 = 10000000;
	if ((self->f_L1->Il > 0.0000001) || (self->f_L1->Il < -0.0000001)) {
	    R_L1 = (self->f_C1->Uc - self->f_C4->Uc) / self->f_L1->Il;
	    if (R_L1 < 0) R_L1 = -R_L1;
	    if (R_L1 < 0.001) R_L1 = 0.001;
	} */

	I_RG = (ug - self->f_C1->Uges) / Rg;
	I_RG -= self->f_L1->Iges;

//	self->f_C1->Uc += I_RG * R_C1;
	self->f_C1->Uges += I_RG * R_C1;

	U = self->f_C1->Uges - self->f_C4->Uges;
	if (self->f_L1->cbOptOn->Checked && self->f_L1->cbRs->Checked) {
	     U -= self->f_L1->Il * self->f_L1->Rs->dval;
	}
	self->f_L1->Il += U * ps_add / (self->f_L1->TL->dval * 1000000);




	if (self->f_L1->cbOptOn->Checked && self->f_L1->cbRp->Checked) {
	    I += (self->f_C1->Uges - self->f_C4->Uges) / (self->f_L1->Rp->dval * 1000);
	}
	I += self->f_L1->Il;
	self->f_L1->Iges = I;

	I -= self->f_C4->Uges / self->f_Ra->TR->dval;

	self->f_C4->Uges += I * R_C4;


//	i = (ug - self->f_C1->Uc) / self->f_Rg->TR->dval;
//	self->f_C1->Uc += i * ps_add / self->f_C1->TC->dval;
	res = self->f_C4->Uges;
#endif

#else

  double i,U;

	i = (ug - self->f_C1->Uc) / self->f_Rg->TR->dval;
	i -= self->f_L1->Il;
	self->f_C1->Uc += i * ps_add / self->f_C1->TC->dval;

	U = self->f_C1->Uc - self->f_C4->Uc;

	if (self->f_L1->cbOptOn->Checked && self->f_L1->cbRp->Checked) {
	    self->f_L1->Il += U / (self->f_L1->Rp->dval * 1000);
	}

	if (self->f_L1->cbOptOn->Checked && self->f_L1->cbRs->Checked) {
	     U -= self->f_L1->Il * self->f_L1->Rs->dval;
	}
	self->f_L1->Il += U * ps_add / (self->f_L1->TL->dval * 1000000);

//	u2 = self->f_L1->Il * self->f_Ra->TR->dval;
//	i = (u2 - self->f_C4->Uc) / self->f_Ra->TR->dval;
//	self->f_C4->Uc += i * ps_add / self->f_C4->TC->dval;
	i = self->f_C4->Uc / self->f_Ra->TR->dval;

	self->f_C4->Uc += (self->f_L1->Il - i)* ps_add / self->f_C4->TC->dval;

	res = self->f_C4->Uc;
#endif
	return res;
}

#endif

static void math_Filter(TPage_TP1 *self) {
  TPage_Filter *pgf = self->pgfilt;
  float fcStep, fOff, fStep;
  TScale_pxy *pxy = pgf->xy_ind->dat->pxy;
  TScale_pxy *pxy2;
  TScale_pxy *pxy3 = NULL;
  float fc, Z, K, ug;
  TScale_Point *sSP, *dSP;
#ifdef FILTER_PULS
  float ps_add;
#endif

	K = 1;
	if (pgf->cbRgRa->Checked)
	    K = (self->f_Rg->rlc.val + self->f_Ra->rlc.val) / self->f_Ra->rlc.val;

	fOff = pgf->ScaleMHzoff;
	fStep = pgf->ScaleMHz;
	fcStep = fStep / (pxy->count-1);
	for (int i=0;i<pxy->count;i++) {
	    fc = i * fcStep + fOff;
	    pxy->xy[i].x = fc;
	    pxy->xy[i].y = K * ImpF1(self, fc, &Z);
	    pxy->xy[i].y2 = Z;
	}
	if (self->cbF1ImpOn->Checked) pxy3 = pxy;

	pxy = pxy->next;
	fcStep = fStep / (pxy->count-1);
	for (int i=0;i<pxy->count;i++) {
	    fc = i * fcStep + fOff;
	    pxy->xy[i].x = fc;
	    pxy->xy[i].y = K * ImpF2(self, fc, &Z);
	    pxy->xy[i].y2 = Z;
	}
	if (self->cbF2ImpOn->Checked) pxy3 = pxy;

	pxy = pxy->next;
	fcStep = fStep / (pxy->count-1);
	for (int i=0;i<pxy->count;i++) {
	    fc = i * fcStep + fOff;
	    pxy->xy[i].x = fc;
	    pxy->xy[i].y = K * ImpF3(self, fc, &Z);
	    pxy->xy[i].y2 = Z;
	}
	if (self->cbF3ImpOn->Checked) pxy3 = pxy;


	pxy = pxy->next;
	fcStep = fStep / (pxy->count-1);
	for (int i=0;i<pxy->count;i++) {
	    fc = i * fcStep + fOff;
	    pxy->xy[i].x = fc;
	    pxy->xy[i].y = K * ImpF4(self, fc, &Z);
	    pxy->xy[i].y2 = Z;
	}
	if (self->cbF4ImpOn->Checked) pxy3 = pxy;

	Z = self->dv_Rg->dval;
	if (pxy3 && (Z > 0)) {
	    pxy2 = pgf->yInZ;
	    pxy2->PaintDat = true;
	    sSP = &pxy3->xy[0];
	    dSP = &pxy2->xy[0];
	    for (int i=0;i<pxy2->count;i++) {
		dSP->x = sSP->x;
		dSP->y = sSP->y2 / Z;
		dSP++; sSP++;
	    }
	} else {
	    pgf->yInZ->PaintDat = false;
	}

#ifdef FILTER_PULS
	ps_add = 100;
	XF_PulsClr((TPage_XFilter*)self, ps_add);
	Form_C_Clr(self->f_C1);
	Form_C_Clr(self->f_C4);
	Form_L_Clr(self->f_L1, ps_add);
	ug = 1.0;


   if (XXXXX) { 
    XXXXX = 0;
#if 0
	    PulsF2(self, ug, ps_add);
	    PulsF2(self, ug, ps_add);
	    PulsF2(self, ug, ps_add);
	    PulsF2(self, ug, ps_add);
/*	    PulsF2(self, 0, 1000);
	    PulsF2(self, 0, 1000);
	    PulsF2(self, 0, 1000);
	    PulsF2(self, 0, 1000);
	    PulsF2(self, 0, 1000);
	    PulsF2(self, 0, 1000);
	    PulsF2(self, 0, 1000);
	    PulsF2(self, 0, 1000);
	    PulsF2(self, 0, 1000);
	    PulsF2(self, 0, 1000);
	    PulsF2(self, 0, 1000);
	    PulsF2(self, 0, 1000);
	    PulsF2(self, 0, 1000);
	    PulsF2(self, 0, 1000);
	    PulsF2(self, 0, 1000); */
#else
	fcStep = fStep / (pxy->count-1);
	pxy = pgf->yPulsZ;
	pxy->PaintDat = true;
	for (int i=0;i<pxy->count;i++) {
	    fc = i * fcStep + fOff;
	    pxy->xy[i].x = fc;
	    pxy->xy[i].y = (float)i / pxy->count;

//	    if (i >= 500) ug = 0;
	    pxy->xy[i].y = PulsF2(self, ug, ps_add);
	    pxy->xy[i].y2 = Z;
	}
#endif
//	    pgf->yPulsZ->PaintDat = false;
   }
	XXXXX++;
#endif

	SetCursor_Dat_txt(pgf);
	Refresh((TWins*)pgf->xy_ind);

}

void TP1_SetVisible(TPage_TP1 *self, bool On) {
	SetVisible((TWins*)self->f_C1, self->f_C1->tag && On);
	SetVisible((TWins*)self->f_C2, self->f_C2->tag && On);
	SetVisible((TWins*)self->f_C3, self->f_C3->tag && On);
	SetVisible((TWins*)self->f_C4, self->f_C4->tag && On);
	SetVisible((TWins*)self->f_L1, self->f_L1->tag && On);
	SetVisible((TWins*)self->f_L2, self->f_L2->tag && On);
	SetVisible((TWins*)self->f_L3, self->f_L3->tag && On);

	XF_SetVisible((TPage_XFilter*)self, On);
}

/* ************** TP1 Ext Ev ****************** */
static bool MathImp(TPage_TP1 *self, TMgsMathImp *pImp) {
	if (!pImp) return false;
	if (self->cbF1ImpOn->Checked) {
	    pImp->yVal = ImpF1(self, pImp->MHz, &pImp->Z);
	    return true;
	} else if (self->cbF2ImpOn->Checked) {
	    pImp->yVal = ImpF2(self, pImp->MHz, &pImp->Z);
	    return true;
	} else if (self->cbF3ImpOn->Checked) {
	    pImp->yVal = ImpF3(self, pImp->MHz, &pImp->Z);
	    return true;
	} else if (self->cbF4ImpOn->Checked) {
	    pImp->yVal = ImpF4(self, pImp->MHz, &pImp->Z);
	    return true;
	}
	return false; 
}

static void DVal_Update(TFormRLC *rlc, double val) {
	if (rlc->Update) rlc->Update(rlc, val, 0);
	if (rlc->SaveData) rlc->SaveData(rlc, NULL);
}

static bool self_OnMsg(TPage_TP1 *self, int msg, void *p) {
  int i;
  TpcIni *pcIni;
  TFormSetLC0 *LC0;

	switch (msg & MSG_BEF_MASK) {
	case MSG_MATH_FILTER:
	    math_Filter(self);
	    return true;

	case MSG_MATH_IMP:
	    return MathImp(self, p);

	case MSG_NEW_SCALE_PARAM:
	    pcIni = get_appIni();
	    i = 1;
	    if (self->ScaleMHz >= 1) i = 0;
	    TpcIni_WR_float(pcIni, self->gname, "ScaleMHz", self->ScaleMHz, i);
	    TpcIni_Save(pcIni);
	    free_appIni(pcIni);
	    return true;

	case MSG_SetLC0:
	    if (!p) return false;
	    LC0 = p;
	    i = 0;
	    if (LC0->UseLsLx) {
		i = 1;
		DVal_Update((TFormRLC*)self->f_Ls, LC0->L * LC0->v[0]);
		DVal_Update((TFormRLC*)self->f_Lx, LC0->L * LC0->v[LC0->vi - 1]);
		TChkBox_SetChecked(self->cbLsOn, true);
		TChkBox_SetChecked_ev(self->cbLxOn, true);
	    } else {
		TChkBox_SetChecked(self->cbLsOn, false);
		TChkBox_SetChecked_ev(self->cbLxOn, false);
	    }
	    switch (LC0->nr) {
	    case 1:
		DVal_Update((TFormRLC*)self->f_C2, LC0->C * LC0->v[i]);
		math_Filter(self);
		return true;
	    case 2:
		DVal_Update((TFormRLC*)self->f_C1, LC0->C * LC0->v[i++]);
		DVal_Update((TFormRLC*)self->f_L1, LC0->L * LC0->v[i++]);
		DVal_Update((TFormRLC*)self->f_C4, LC0->C * LC0->v[i]);
		math_Filter(self);
		return true;
	    case 3:
		DVal_Update((TFormRLC*)self->f_C1, LC0->C * LC0->v[i++]);
		DVal_Update((TFormRLC*)self->f_L1, LC0->L * LC0->v[i++]);
		DVal_Update((TFormRLC*)self->f_C2, LC0->C * LC0->v[i++]);
		DVal_Update((TFormRLC*)self->f_L3, LC0->L * LC0->v[i++]);
		DVal_Update((TFormRLC*)self->f_C4, LC0->C * LC0->v[i]);
		math_Filter(self);
		return true;
	    case 4:
		DVal_Update((TFormRLC*)self->f_C1, LC0->C * LC0->v[i++]);
		DVal_Update((TFormRLC*)self->f_L1, LC0->L * LC0->v[i++]);
		DVal_Update((TFormRLC*)self->f_C2, LC0->C * LC0->v[i++]);
		DVal_Update((TFormRLC*)self->f_L2, LC0->L * LC0->v[i++]);
		DVal_Update((TFormRLC*)self->f_C3, LC0->C * LC0->v[i++]);
		DVal_Update((TFormRLC*)self->f_L3, LC0->L * LC0->v[i++]);
		DVal_Update((TFormRLC*)self->f_C4, LC0->C * LC0->v[i]);
		math_Filter(self);
		return true;
	    }
	    return false;
	}
	return false;
}

/* ************** TP1 Ev ****************** */
static void cbXxOn_OnClick(TChkBox *self, long KeyState, bool DblClick) {
  TPage_TP1 *pg = self->subClass;
  TpcIni *pcIni = TpcIni_Create(appDat.inifile_app);
	XF_SaveOnOff((TPage_XFilter*)pg, pcIni);
	TpcIni_SaveAndFree(pcIni);
	RePaint((TWins*)pg->pnl);
	math_Filter(pg);
}

static void cbFxOn_OnClick(TChkBox *self, long KeyState, bool DblClick) {
  TPage_TP1 *pg = self->subClass;
//  TPage_Filter *pg = self->subClass;
  TpcIni *pcIni = TpcIni_Create(appDat.inifile_app);
	XF_SaveOnOff((TPage_XFilter*)pg, pcIni);
	TpcIni_SaveAndFree(pcIni);
	((TScale_pxy*)self->tag)->PaintDat = ((TChkBox*)self)->Checked;
	Refresh((TWins*)pg->pgfilt->xy_ind);
}

static void cbFxImpOn_OnClick(TChkBox *self, long KeyState, bool DblClick) {
  TPage_TP1 *pg = self->subClass;
  TpcIni *pcIni = TpcIni_Create(appDat.inifile_app);
	if (pg->cbF1ImpOn != self) TChkBox_SetChecked(pg->cbF1ImpOn, false);
	if (pg->cbF2ImpOn != self) TChkBox_SetChecked(pg->cbF2ImpOn, false);
	if (pg->cbF3ImpOn != self) TChkBox_SetChecked(pg->cbF3ImpOn, false);
	if (pg->cbF4ImpOn != self) TChkBox_SetChecked(pg->cbF4ImpOn, false);
	XF_SaveOnOff((TPage_XFilter*)pg, pcIni);
	TpcIni_SaveAndFree(pcIni);
	math_Filter(pg);
	Refresh((TWins*)pg->pgfilt->xy_ind);
}

static void pnl_OnPaint(TWins *self) {
  TPage_TP1 *pg = self->subClass;
  int fu, x;

	fu = XF_GetPaint_Fu((TPage_XFilter*)pg);
	x = pg->cbF1On->left + pg->cbF1On->width + 2;

	DrawSch(self->WinID, x, pg->cbF1On->top, fu);
	fu += 0x100;
	DrawSch(self->WinID, x, pg->cbF2On->top, fu);
	fu += 0x100;
	DrawSch(self->WinID, x, pg->cbF3On->top, fu);
	fu += 0x100;
	DrawSch(self->WinID, x, pg->cbF4On->top, fu);

	DrawPic(self->WinID, get_Pic_cOhm(), 
			pg->dv_Ra->left + pg->dv_Ra->width + 2,
			pg->dv_Ra->top + 5);

	str_toGraf(self, "pF", pg->dv_Cx->left + pg->dv_Cx->width + 1,
			       pg->dv_Cx->top + 3);

	str_toGraf(self, "uH", pg->dv_Lx->left + pg->dv_Lx->width + 1,
			       pg->dv_Lx->top + 3);

	str_toGraf(self, "pF", pg->dv_C4->left + pg->dv_C4->width + 1,
			       pg->dv_C4->top + 3);

	str_toGraf(self, "uH", pg->dv_L3->left + pg->dv_L3->width + 1,
			       pg->dv_L3->top + 3);
}

static void TP1_DVal_OnValChange(TDVal *dv, double val) {
	if (dv->owner) TDVal_DoValChange((TDVal*)dv->owner, val, tdvfu_ChangeEv);
}

static void TP1_fXxx_OnChange(TFormRLC* fRLC) {
	math_Filter((TPage_TP1*)fRLC->subClass);
}
/* static void TP1_OnRSync(TFormRLC* fRLC, int val) {
  TPage_TP1 *pg = fRLC->subClass;
	Form_R_sync(fRLC, pg->f_Rg, val);
	Form_R_sync(fRLC, pg->f_Rs, val);
	Form_R_sync(fRLC, pg->f_Rx, val);
	Form_R_sync(fRLC, pg->f_Ra, val);
} */
static void TP1_OnLSync(TFormRLC* fRLC, int val) {
  TPage_TP1 *pg = fRLC->subClass;
	Form_L_sync(fRLC, pg->f_L1, val);
	Form_L_sync(fRLC, pg->f_L2, val);
	Form_L_sync(fRLC, pg->f_L3, val);
}
static void TP1_OnCSync(TFormRLC* fRLC, int val) {
  TPage_TP1 *pg = fRLC->subClass;
	Form_C_sync(fRLC, pg->f_C1, val);
	Form_C_sync(fRLC, pg->f_C2, val);
	Form_C_sync(fRLC, pg->f_C3, val);
	Form_C_sync(fRLC, pg->f_C4, val);
}
/* ************** Save Load ****************** */
static void SaveData(TPage_TP1 *self, TpcIni *ini) {
	XF_SaveData((TPage_XFilter*)self, ini);

	self->f_L1->SaveData(self->f_L1, ini);
	self->f_L2->SaveData(self->f_L2, ini);
	self->f_L3->SaveData(self->f_L3, ini);

	self->f_C1->SaveData(self->f_C1, ini);
	self->f_C2->SaveData(self->f_C2, ini);
	self->f_C3->SaveData(self->f_C3, ini);
	self->f_C4->SaveData(self->f_C4, ini);
}

static void LoadData(TPage_TP1 *self, TpcIni *ini) {

	XF_LoadData((TPage_XFilter*)self, ini);

	self->f_L1->LoadData(self->f_L1, ini);
	self->f_L2->LoadData(self->f_L2, ini);
	self->f_L3->LoadData(self->f_L3, ini);

	self->f_C1->LoadData(self->f_C1, ini);
	self->f_C2->LoadData(self->f_C2, ini);
	self->f_C3->LoadData(self->f_C3, ini);
	self->f_C4->LoadData(self->f_C4, ini);

	math_Filter(self);
	Refresh((TWins*)self->pgfilt->xy_ind);
}

/* ************** TP1 Create ****************** */
static void DVal_Create_Xxx(TWins *Parent, TDVal **dv, TFormRLConoff **onoff, int *x, int y, int w, int nk, TPic *pic) {
  int tx = *x;
  TDVal *tdv = TDVal_CreateS(Parent, tx+19, y, w, 19);
	tdv->nk = nk;
	tdv->dv_fu = tdv_fu_LostFocusEnter;
	tdv->OnValChange = TP1_DVal_OnValChange;
	*dv = tdv;
	*onoff = TFormRLConoff_CreateU(Parent, tx, y+1, 18, 16, pic);
	*x = tx + w + 21;
}

/* static TFormR *Create_fR(TPage_TP1 *self, const char *Caption,
			const char *s, TDVal *dv, float val) {
  TFormR *fR = Create_Form_R(appDat.obj, Caption, self->gname, s, dv, val);
	fR->subClass = self;
	TDVal_Set_Value(dv, fR->R->dval);
	fR->OnChange = TP1_fXxx_OnChange;
	fR->OnSync = TP1_OnRSync;
	return fR;
} */
static TFormL *Create_fL(TPage_TP1 *self, const char *Caption,
			const char *s, TDVal *dv, float val) {
  TFormL *fL = Create_Form_L(appDat.obj, Caption, self->gname, s, dv, val);
	fL->subClass = self;
	TDVal_Set_Value(dv, fL->L->dval);
	fL->OnChange = TP1_fXxx_OnChange;
	fL->OnSync = TP1_OnLSync;
	return fL;
}
static TFormC *Create_fC(TPage_TP1 *self, const char *Caption,
			const char *s, TDVal *dv, float val) {
  TFormC *fC = Create_Form_C(appDat.obj, Caption, self->gname, s, dv, val);
	fC->subClass = self;
	TDVal_Set_Value(dv, fC->C->dval);
	fC->OnChange = TP1_fXxx_OnChange;
	fC->OnSync = TP1_OnCSync;
	return fC;
}

TPage_TP1 *page_Add_TP1(TPageControl *pagc, TPage_Filter *pgfilt) {
  TPage_TP1 *self;
  TWins *pnl;
  int x;
  TAddXFilter xf;

	xf.pagc = pagc;
	xf.pgfilt = pgfilt;
	xf.gname = "TP1";
	xf.psize = sizeof(TPage_TP1);
	xf.ini = get_appIni();
	xf.TDValOnValChange = TP1_DVal_OnValChange;
	xf.fXxx_OnChange = TP1_fXxx_OnChange;
	xf.OnRSync = NULL; //TP1_OnRSync;
	xf.OnLSync = TP1_OnLSync;
	xf.OnCSync = TP1_OnCSync;
	xf.dvfu = FU_RG|FU_RS|FU_RX|FU_RA|FU_CS|FU_CX|FU_LS|FU_LX;

	xf.cb_OnClick = cbXxOn_OnClick;
	xf.cbfu = FU_RS|FU_RX|FU_CS|FU_CX|FU_LS|FU_LX;
	xf.cbOnOff = 0xffe0;
	xf.cbY = 82;

	xf.cbFxOnClick = cbFxOn_OnClick;
	xf.cbFxImpOnClick = cbFxImpOn_OnClick;
	xf.cbF1Y = 105;
	xf.cbF2Y = 150;
	xf.cbF3Y = 195;
	xf.cbF4Y = 240;
	xf.cbFxfu = FU_F1|FU_F2|FU_F3|FU_F4;

	xf.LC0_Popupfu = FU_F1|FU_F2|FU_F3|FU_F4;

	self = (TPage_TP1*)PageAddXFilter(&xf);
	self->pnl->OnPaint = pnl_OnPaint;
	pnl = self->pnl;
/* C1 C2 C3 */
	x = 0;
	DVal_Create_Xxx(pnl, &self->dv_C1, &self->OnOff_C1, &x, 42, 54, 1, get_Pic_cC1());
	DVal_Create_Xxx(pnl, &self->dv_C2, &self->OnOff_C2, &x, 42, 54, 1, get_Pic_cC2());
	DVal_Create_Xxx(pnl, &self->dv_C3, &self->OnOff_C3, &x, 42, 54, 1, get_Pic_cC3());
	DVal_Create_Xxx(pnl, &self->dv_C4, &self->OnOff_C4, &x, 42, 54, 1, get_Pic_cC4());
/* L1 L2 L3 */
	x = 0;
	DVal_Create_Xxx(pnl, &self->dv_L1, &self->OnOff_L1, &x, 62, 54, 3, get_Pic_cL1());
	DVal_Create_Xxx(pnl, &self->dv_L2, &self->OnOff_L2, &x, 62, 54, 3, get_Pic_cL2());
	DVal_Create_Xxx(pnl, &self->dv_L3, &self->OnOff_L3, &x, 62, 54, 3, get_Pic_cL3());

	self->f_C1 = Create_fC(self, "TP1 C1", "C1", self->dv_C1, 8);
	self->f_C2 = Create_fC(self, "TP1 C2", "C2", self->dv_C2, 16);
	self->f_C3 = Create_fC(self, "TP1 C3", "C3", self->dv_C3, 16);
	self->f_C4 = Create_fC(self, "TP1 C4", "C4", self->dv_C4, 8);

	self->f_L1 = Create_fL(self, "TP1 L1", "L1", self->dv_L1, 0.027);
	self->f_L2 = Create_fL(self, "TP1 L2", "L2", self->dv_L2, 0.027);
	self->f_L3 = Create_fL(self, "TP1 L3", "L3", self->dv_L3, 0.027);

	TFormRLConoff_SetForm(self->OnOff_C1, (TForm*)self->f_C1);
	TFormRLConoff_SetForm(self->OnOff_C2, (TForm*)self->f_C2);
	TFormRLConoff_SetForm(self->OnOff_C3, (TForm*)self->f_C3);
	TFormRLConoff_SetForm(self->OnOff_C4, (TForm*)self->f_C4);
	TFormRLConoff_SetForm(self->OnOff_L1, (TForm*)self->f_L1);
	TFormRLConoff_SetForm(self->OnOff_L2, (TForm*)self->f_L2);
	TFormRLConoff_SetForm(self->OnOff_L3, (TForm*)self->f_L3);

	self->OnMsg = self_OnMsg;

	self->SaveData = SaveData;
	self->LoadData = LoadData;

	free_appIni(xf.ini);

	return self;
}
