/*
 Copyright (C) 2010-2017 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

#include "ElMath.h"
#include "ElGraf.h"
#include <math.h>

static void str_toGraf(TWins *Wins, const char *s, int x, int y) {
	vds_SetStr(&Wins->vds, s);
	DrawColorString(x, y, &Wins->vds);
}

static void DrawSch(Window WinID, int x, int y, int fu) {
  int xa;
  int x2 = x;
  int y2 = y+30;

	Draw_xP1_Rg(WinID, &x2, &xa, y, y2, fu);

	if ((fu & 0x0300) == 0x000) {
	    DrawTP1_Cv(WinID, &x2, 0, y, get_Pic_cC2());
	}

	if ((fu & 0x0300) == 0x100) {
	    DrawTP1_Cv(WinID, &x2, 0, y, get_Pic_cC1());
	    DrawTP1_Lh(WinID, &x2, y, get_Pic_cL1());
	    DrawTP1_Cv(WinID, &x2, 0, y, get_Pic_cC4());
	}
	if ((fu & 0x0300) == 0x200) {
	    DrawTP1_Cv(WinID, &x2, 0, y, get_Pic_cC1());
	    DrawTP1_Lh(WinID, &x2, y, get_Pic_cL1());
	    DrawTP1_Cv(WinID, &x2, 0, y, get_Pic_cC2());
	    DrawTP1_Lh(WinID, &x2, y, get_Pic_cL3());
	    DrawTP1_Cv(WinID, &x2, 0, y, get_Pic_cC4());
	}

	if ((fu & 0x0300) == 0x300) {
	    DrawTP1_Cv(WinID, &x2, 0, y, get_Pic_cC1());
	    DrawTP1_Lh(WinID, &x2, y, get_Pic_cL1());
	    DrawTP1_Cv(WinID, &x2, 0, y, get_Pic_cC2());
	    DrawTP1_Lh(WinID, &x2, y, get_Pic_cL2());
	    DrawTP1_Cv(WinID, &x2, 0, y, get_Pic_cC3());
	    DrawTP1_Lh(WinID, &x2, y, get_Pic_cL3());
	    DrawTP1_Cv(WinID, &x2, 0, y, get_Pic_cC4());
	}

	Draw_xP1_Ra(WinID, x2, xa, y, y2, fu);
}

static float ImpF1(TPage_TP1 *self, float MHz, float *Z) {
  float res;
  TRLC btIn;
  float w = MHz * M_PI * 2;
	if (!XF_MathOut((TPage_XFilter*)self, w, &btIn, &res)) return res;
	Form_C_math(self->f_C2, w);
	RLC_Add_Par(&btIn, &self->f_C2->rlc);
	res *= btIn.Z;

	XF_MathIn((TPage_XFilter*)self, w, &btIn, &res, Z);
	return res;
} 
static float ImpF2(TPage_TP1 *self, float MHz, float *Z) {
  float res;
  TRLC btIn;
  float w = MHz * M_PI * 2;
	if (!XF_MathOut((TPage_XFilter*)self, w, &btIn, &res)) return res;
	Form_C_math(self->f_C1, w);
	Form_L_math(self->f_L1, w);
	Form_C_math(self->f_C4, w);
	RLC_Add_Par(&btIn, &self->f_C4->rlc);
	res *= btIn.Z;
	RLC_Add_Ser(&btIn, &self->f_L1->rlc);
	res /= btIn.Z;
	RLC_Add_Par(&btIn, &self->f_C1->rlc);
	res *= btIn.Z;

	XF_MathIn((TPage_XFilter*)self, w, &btIn, &res, Z);
	return res;
}
static float ImpF3(TPage_TP1 *self, float MHz, float *Z) {
  float res;
  TRLC btIn;
  float w = MHz * M_PI * 2;
	if (!XF_MathOut((TPage_XFilter*)self, w, &btIn, &res)) return res;
	Form_C_math(self->f_C1, w);
	Form_C_math(self->f_C2, w);
	Form_L_math(self->f_L1, w);
	Form_L_math(self->f_L3, w);
	Form_C_math(self->f_C4, w);
	RLC_Add_Par(&btIn, &self->f_C4->rlc);
	res *= btIn.Z;
	RLC_Add_Ser(&btIn, &self->f_L3->rlc);
	res /= btIn.Z;
	RLC_Add_Par(&btIn, &self->f_C2->rlc);
	res *= btIn.Z;
	RLC_Add_Ser(&btIn, &self->f_L1->rlc);
	res /= btIn.Z;
	RLC_Add_Par(&btIn, &self->f_C1->rlc);
	res *= btIn.Z;

	XF_MathIn((TPage_XFilter*)self, w, &btIn, &res, Z);
	return res;
}
static float ImpF4(TPage_TP1 *self, float MHz, float *Z) {
  float res;
  TRLC btIn;
  float w = MHz * M_PI * 2;
	if (!XF_MathOut((TPage_XFilter*)self, w, &btIn, &res)) return res;
	Form_C_math(self->f_C1, w);
	Form_C_math(self->f_C2, w);
	Form_C_math(self->f_C3, w);
	Form_L_math(self->f_L1, w);
	Form_L_math(self->f_L2, w);
	Form_L_math(self->f_L3, w);
	Form_C_math(self->f_C4, w);
	RLC_Add_Par(&btIn, &self->f_C4->rlc);
	res *= btIn.Z;
	RLC_Add_Ser(&btIn, &self->f_L3->rlc);
	res /= btIn.Z;
	RLC_Add_Par(&btIn, &self->f_C3->rlc);
	res *= btIn.Z;
	RLC_Add_Ser(&btIn, &self->f_L2->rlc);
	res /= btIn.Z;
	RLC_Add_Par(&btIn, &self->f_C2->rlc);
	res *= btIn.Z;
	RLC_Add_Ser(&btIn, &self->f_L1->rlc);
	res /= btIn.Z;
	RLC_Add_Par(&btIn, &self->f_C1->rlc);
	res *= btIn.Z;

	XF_MathIn((TPage_XFilter*)self, w, &btIn, &res, Z);
	return res;
}

#ifdef FILTER_PULS
static int XXXXX;
static bool Ptgr;

static double ParR(double R1, double R2) {
//	if (R1 < 0) R1 = -R1;
//	if (R2 < 0) R2 = -R2;
	return 1 / ((1 / R1) + (1 / R2));
}
static double GetRU(double U, double R1, double R2) {
	return U / ((R1 / R2) + 1);
}

static float PulsF1(TPage_TP1 *self, float ug) {
  float res;

  double U, R, RG, RA; //, RGC1, RAC4;

/*
  int ipr;
  XXXXX++;
  ipr = 0;
  if (!(XXXXX & 0x0f) && (XXXXX <= 640)) ipr = XXXXX / 8;
*/
	RG = self->f_Rg->TR->dval;
	RA = self->f_Ra->TR->dval;

	if ((self->cbLsOn && self->cbLsOn->Checked) && 
	    (self->cbLxOn && self->cbLxOn->Checked)) {

	    Form_L_Puls_math(self->f_Ls, RG, ug - self->f_C2->U);

	    R = RG * RA / (RG + RA);
	    U = (ug - self->f_Ls->U + self->f_Lx->U) / ((RG / RA) + 1);

	    Form_C_Puls_math(self->f_C2, R, U);

	    Form_L_Puls_math(self->f_Lx, RA, self->f_C2->U);
	    res = Form_L_Puls_get_U(self->f_Lx, RA);
// if (ipr) printf("%4d %9.3f %9.4f   U1=%9.4f I1=%9.4f\n", ipr, ug, res, self->f_C2->U, self->f_C2->I);

	} else if (self->cbLsOn && self->cbLsOn->Checked) {

	    U = ug - self->f_C2->U;
	    Form_L_Puls_math(self->f_Ls, RG, U);
	    U = Form_L_Puls_get_U(self->f_Ls, RA);
// CXXX += (self->f_Ls->coL.I - (self->f_C2->U / RA)) * self->f_C2->RC;


	    Form_C_Puls_math(self->f_C2, RA, U);
	    res = self->f_C2->U;

// if (ipr) printf("%4d %9.3f %9.4f U=%9.3f I=%9.3f\n", ipr, ug, res, self->f_Ls->U, self->f_Ls->I * 1000.0);

	} else if (self->cbLxOn && self->cbLxOn->Checked) {

	    U = ug - Form_L_Puls_get_U(self->f_Lx, RG);
	    Form_C_Puls_math(self->f_C2, RG, U);
	    Form_L_Puls_math(self->f_Lx, RA, self->f_C2->U);
	    res = Form_L_Puls_get_U(self->f_Lx, RA);

	} else {
// double Iadd;
	    U = ug / ((RG / RA) + 1);
	    R = RG * RA / (RG + RA);
//	Iadd = self->f_C2->coC.I;
	    Form_C_Puls_math(self->f_C2, R, U);
	    res = self->f_C2->U;
//	Iadd = self->f_C2->coC.I - Iadd;
//	R = self->f_C2->coC.U / self->f_C2->coC.I;

// if (ipr) printf("%4d %9.3f %9.4f I=%9.5f R=%9.3f I=%9.5f\n", ipr, ug, res, self->f_C2->coC.I * 1000.0, R, self->f_C2->coLs.I * 1000.0);
	}

	return res;
}

static void PulsTrg_LH_F2(TPage_TP1 *self, float ug) {
  float R, R1, U1, R2, U;

	R1 = self->f_Rg->TR->dval;
	U1 = ug;
	if (self->f_C1->rlcex & RLC_RP) {
	    R = self->f_C1->Rp->dval * 1000;
	    U1 = GetRU(U1, R1, R);
	    R1 = ParR(R1, R);
	}
	if (self->f_L1->rlcex & RLC_RS) R1 += self->f_L1->Rs->dval;


	R2 = self->f_Ra->TR->dval;
	if (self->f_C4->rlcex & RLC_RP) {
	    R2 = ParR(R2, self->f_C4->Rp->dval * 1000);
	}

	U = GetRU(U1, R1, R2);

//  printf("U1=%6.3f R1=%6.3f R2=%6.3f U=%6.3f\n", U1, R1, R2, U);
	TFormPuls_SetTrigV(self->f_Puls, U, TRG_LH);
}

static float PulsF2(TPage_TP1 *self, float ug) {
  float res;
/*
  int ipr;
  XXXXX++;
  ipr = 0;
  if (!(XXXXX & 0x0f) && (XXXXX <= 640)) ipr = XXXXX / 8;
*/
//  if ((XXXXX >= 40) && (XXXXX <= 80)) ipr = XXXXX;

// if (XXXXX == 1) printf("%9.3f %9.3f %9.3f\n", 
//			self->f_C1->Puls_Z, self->f_L1->Puls_Z, self->f_C4->Puls_Z);

#if 1
  double RG, RA, UC1, UL1, UC4;

	RG = self->f_Rg->TR->dval;
	RA = self->f_Ra->TR->dval;

	UC1 = ug - Form_L_Puls_get_U(self->f_L1, RG);
	Form_C_Puls_math(self->f_C1, RG, UC1);

	UL1 = ug - (self->f_C1->I * RG) - self->f_C4->U;
	Form_L_Puls_math(self->f_L1, RG, UL1);

	UC4 = Form_L_Puls_get_U(self->f_L1, RA);
	Form_C_Puls_math(self->f_C4, RA, UC4);

	res = self->f_C4->U;
/*
 if (ipr) printf("F2 %4d %9.3f %9.4f   U1=%9.4f I=%9.4f/%9.4f\n", 
	ipr, ug, res, self->f_C1->U, self->f_C1->coC.I * 1000.0, self->f_C1->coLs.I * 1000.0);
*/

#else
  double U, R, RG, RA; //, RGC1, RAC4;

	RG = self->f_Rg->TR->dval;
	RA = self->f_Ra->TR->dval;

	if (self->f_L1->rlcex & RLC_RP) {
//	    U = GetRU(ug, RG, self->f_L1->RL_Rp + RA);

//	    U = (self->f_C1->U - self->f_C4->U);

	    U = ug - Form_L_Puls_get_U(self->f_L1, RG); //(RG * self->f_L1->I);

//	    R = ParR(RG, self->f_L1->RL_Rp);
	    Form_C_Puls_math(self->f_C1, RG, U);

//	    R = ParR(RG, self->f_C1->RC) + ParR(RA, self->f_C4->RC);
	    R = 0.0001;
	    Form_L_Puls_math(self->f_L1, R, self->f_C1->U - self->f_C4->U);

	
//	    R = ParR(RA, self->f_L1->RL_Rp);
//	    Form_C_Puls_math(self->f_C4, RA, self->f_L1->I * RA);
	    Form_C_Puls_math(self->f_C4, RA, Form_L_Puls_get_U(self->f_L1, RA));

	} else {

//	RGC1 = ParR(RG, self->f_C1->RC);
//	RAC4 = ParR(RA, self->f_C4->RC);

#if FILTER_PULS_OLD
	    U = ug - (RG * self->f_L1->IL);
#else
	    U = ug - (RG * self->f_L1->coL.I);
#endif
	    Form_C_Puls_math(self->f_C1, RG, U);

	    R = 0.0001;
	    Form_L_Puls_math(self->f_L1, R, self->f_C1->U - self->f_C4->U);

	
#if FILTER_PULS_OLD
	    Form_C_Puls_math(self->f_C4, RA, self->f_L1->IL * RA);
#else
	    Form_C_Puls_math(self->f_C4, RA, self->f_L1->coL.I * RA);
#endif
	}
	res = self->f_C4->U;

#endif

	return res;
}

static float PulsF2in(TPage_TP1 *self, float ug) {
//	return Form_L_Puls_get_U(self->f_L1, self->f_Rg->TR->dval);
	return self->f_C1->U;
}

static float PulsFx(TPage_TP1 *self, float ug) {
  int i;
	i = self->f_Puls->tag;
	if (i == 1) {
	    return PulsF1(self, ug);
	} else if (i == 2) {
	    return PulsF2(self, ug);
	}
	return  0;
}

#endif /* FILTER_PULS */

static void math_Filter(TPage_TP1 *self) {
  TPage_Filter *pgf = self->pgfilt;
  float fcStep, fOff, fStep;
  TScale_pxy *pxy = pgf->xy_ind->dat->pxy;
  TScale_pxy *pxy2;
  TScale_pxy *pxy3 = NULL;
  float fc, Z, K;
  TScale_Point *sSP, *dSP;
#ifdef FILTER_PULS
  float ps_add, U;
  TScale_Point *xyG, *xyI, *xyZ;
#endif

	K = 1;
	if (pgf->cbRgRa->Checked)
	    K = (self->f_Rg->rlc.val + self->f_Ra->rlc.val) / self->f_Ra->rlc.val;

	fOff = pgf->ScaleMHzoff;
	fStep = pgf->ScaleMHz;
	fcStep = fStep / (pxy->count-1);
	for (int i=0;i<pxy->count;i++) {
	    fc = i * fcStep + fOff;
	    pxy->xy[i].x = fc;
	    pxy->xy[i].y = K * ImpF1(self, fc, &Z);
	    pxy->xy[i].y2 = Z;
	}
	if (self->cbF1ImpOn->Checked) pxy3 = pxy;

	pxy = pxy->next;
	fcStep = fStep / (pxy->count-1);
	for (int i=0;i<pxy->count;i++) {
	    fc = i * fcStep + fOff;
	    pxy->xy[i].x = fc;
	    pxy->xy[i].y = K * ImpF2(self, fc, &Z);
	    pxy->xy[i].y2 = Z;
	}
	if (self->cbF2ImpOn->Checked) pxy3 = pxy;

	pxy = pxy->next;
	fcStep = fStep / (pxy->count-1);
	for (int i=0;i<pxy->count;i++) {
	    fc = i * fcStep + fOff;
	    pxy->xy[i].x = fc;
	    pxy->xy[i].y = K * ImpF3(self, fc, &Z);
	    pxy->xy[i].y2 = Z;
	}
	if (self->cbF3ImpOn->Checked) pxy3 = pxy;


	pxy = pxy->next;
	fcStep = fStep / (pxy->count-1);
	for (int i=0;i<pxy->count;i++) {
	    fc = i * fcStep + fOff;
	    pxy->xy[i].x = fc;
	    pxy->xy[i].y = K * ImpF4(self, fc, &Z);
	    pxy->xy[i].y2 = Z;
	}
	if (self->cbF4ImpOn->Checked) pxy3 = pxy;

	Z = self->dv_Rg->dval;
	if (pxy3 && (Z > 0)) {
	    pxy2 = pgf->yInZ;
	    pxy2->PaintDat = true;
	    sSP = &pxy3->xy[0];
	    dSP = &pxy2->xy[0];
	    for (int i=0;i<pxy2->count;i++) {
		dSP->x = sSP->x;
		dSP->y = sSP->y2 / Z;
		dSP++; sSP++;
	    }
	} else {
	    pgf->yInZ->PaintDat = false;
	}

#ifdef FILTER_PULS
	if (self->f_Puls->tag) {

	    ps_add = TFormPuls_get_ps_add(self->f_Puls, pgf->yPulsG->count - 1, 8);
// printf("ps_add=%9.3f\n", ps_add);
	    XF_PulsClr((TPage_XFilter*)self, ps_add);
	    Form_C_Clr(self->f_C1, ps_add);
	    Form_C_Clr(self->f_C2, ps_add);
	    Form_C_Clr(self->f_C4, ps_add);
	    Form_L_Clr(self->f_Ls, ps_add);
	    Form_L_Clr(self->f_L1, ps_add);
	    Form_L_Clr(self->f_Lx, ps_add);
/*
 if (self->f_Puls->tag == 1) {
 printf("Ls=%9.3f R=%9.3f\n", self->f_C2->Ls->dval, self->f_C2->coLs.Rt);
 printf("C2=%9.3f R=%9.3f\n", self->f_C2->TC->dval, self->f_C2->coC.Rt);
 } else if (self->f_Puls->tag == 2) {
 printf("C1=%9.3f R=%9.3f\n", self->f_C1->TC->dval, self->f_C1->coC.Rt);
 printf("L1=%9.3f R=%9.3f\n", self->f_L1->L->dval, self->f_L1->coL.Rt);
 printf("C4=%9.3f R=%9.3f\n", self->f_C4->TC->dval, self->f_C4->coC.Rt);
 }
*/
// printf("Lx=%9.3f R=%9.3f\n", self->f_C2->Lx->dval, self->f_C2->RC_Lx);


	    PulsTrg_LH_F2(self, self->f_Puls->dvVpp->dval);

	    U = self->f_Puls->Vdiv->dval;
	    pgf->yPulsG->ex->min = -U;
	    pgf->yPulsI->ex->min = -U;
	    pgf->yPulsZ->ex->min = -U;

	    pgf->yPulsG->ex->max = U * 8;
	    pgf->yPulsI->ex->max = U * 8;
	    pgf->yPulsZ->ex->max = U * 8;


   if (XXXXX) { 
    XXXXX = 0;
    Ptgr = false;

	fcStep = fStep / (pxy->count-1);
	pgf->yPulsG->PaintDat = true;
	pgf->yPulsZ->PaintDat = true;
	pxy = pgf->yPulsZ;
	pxy->PaintDat = true;
	

	if (self->f_Puls->cbAzIn->Checked) {
	    pgf->yPulsI->PaintDat = true;
	    xyI = pgf->yPulsI->xy;
	} else {
	    pgf->yPulsI->PaintDat = false;
	    xyI = NULL;
	}
	xyG = pgf->yPulsG->xy;
	xyZ = pgf->yPulsZ->xy;

// dword t;
// t = GetTime_us();
	for (int i=0;i<pxy->count;i++) {
	    fc = i * fcStep + fOff;
	    xyG->x = fc;
	    xyZ->x = fc;
	    if (xyI) xyI->x = fc;

	    PulsFx(self, TFormPuls_get_ampl(self->f_Puls));
	    PulsFx(self, TFormPuls_get_ampl(self->f_Puls));
	    PulsFx(self, TFormPuls_get_ampl(self->f_Puls));

	    U = PulsFx(self, TFormPuls_get_ampl(self->f_Puls));
	    TFormPuls_ChkTrigV(self->f_Puls,  U, i);

	    PulsFx(self, TFormPuls_get_ampl(self->f_Puls));
	    PulsFx(self, TFormPuls_get_ampl(self->f_Puls));
	    PulsFx(self, TFormPuls_get_ampl(self->f_Puls));

	    xyG->y = TFormPuls_get_ampl(self->f_Puls);
	    xyZ->y = PulsFx(self, xyG->y);
	    TFormPuls_ChkTrigV(self->f_Puls, xyZ->y, i);
	    if (xyI) {
		xyI->y = PulsF2in(self, xyG->y); //self->f_C1->U;
		xyI++;
	    }

	    xyZ++;
	    xyG++;
	}
// printf("%dus\n", GetTime_us()-t);

//	    pgf->yPulsZ->PaintDat = false;
   }

	} else {	//if (self->f_Puls->tag)
	    pgf->yPulsG->PaintDat = false;
	    pgf->yPulsI->PaintDat = false;
	    pgf->yPulsZ->PaintDat = false;
	}

	XXXXX++;
#endif

	SetCursor_Dat_txt(pgf);
	Refresh((TWins*)pgf->xy_ind);

}

void TP1_SetVisible(TPage_TP1 *self, bool On) {
	SetVisible((TWins*)self->f_C1, self->f_C1->tag && On);
	SetVisible((TWins*)self->f_C2, self->f_C2->tag && On);
	SetVisible((TWins*)self->f_C3, self->f_C3->tag && On);
	SetVisible((TWins*)self->f_C4, self->f_C4->tag && On);
	SetVisible((TWins*)self->f_L1, self->f_L1->tag && On);
	SetVisible((TWins*)self->f_L2, self->f_L2->tag && On);
	SetVisible((TWins*)self->f_L3, self->f_L3->tag && On);

	XF_SetVisible((TPage_XFilter*)self, On);

#ifdef FILTER_PULS
	SetVisible((TWins*)self->f_Puls, self->f_Puls->tag && On);
#endif /* FILTER_PULS */

}

/* ************** TP1 Ext Ev ****************** */
static bool MathImp(TPage_TP1 *self, TMgsMathImp *pImp) {
	if (!pImp) return false;
	if (self->cbF1ImpOn->Checked) {
	    pImp->yVal = ImpF1(self, pImp->MHz, &pImp->Z);
	    return true;
	} else if (self->cbF2ImpOn->Checked) {
	    pImp->yVal = ImpF2(self, pImp->MHz, &pImp->Z);
	    return true;
	} else if (self->cbF3ImpOn->Checked) {
	    pImp->yVal = ImpF3(self, pImp->MHz, &pImp->Z);
	    return true;
	} else if (self->cbF4ImpOn->Checked) {
	    pImp->yVal = ImpF4(self, pImp->MHz, &pImp->Z);
	    return true;
	}
	return false; 
}

static void DVal_Update(TFormRLC *rlc, double val) {
	if (rlc->Update) rlc->Update(rlc, val, 0);
	if (rlc->SaveData) rlc->SaveData(rlc, NULL);
}

static bool self_OnMsg(TPage_TP1 *self, int msg, void *p) {
  int i;
  TpcIni *pcIni;
  TFormSetLC0 *LC0;

	switch (msg & MSG_BEF_MASK) {
	case MSG_MATH_FILTER:
	    math_Filter(self);
	    return true;

	case MSG_MATH_IMP:
	    return MathImp(self, p);

	case MSG_NEW_SCALE_PARAM:
	    pcIni = get_appIni();
	    i = 1;
	    if (self->ScaleMHz >= 1) i = 0;
	    TpcIni_WR_float(pcIni, self->gname, "ScaleMHz", self->ScaleMHz, i);
	    TpcIni_Save(pcIni);
	    free_appIni(pcIni);
	    return true;

	case MSG_SetLC0:
	    if (!p) return false;
	    LC0 = p;
	    i = 0;
	    if (LC0->UseLs) {
		i = 1;
		DVal_Update((TFormRLC*)self->f_Ls, LC0->L * LC0->v[0]);
//		DVal_Update((TFormRLC*)self->f_Lx, LC0->L * LC0->v[LC0->vi - 1]);
		TChkBox_SetChecked(self->cbLsOn, true);
//		TChkBox_SetChecked_ev(self->cbLxOn, true);
	    } else {
		TChkBox_SetChecked(self->cbLsOn, false);
//		TChkBox_SetChecked_ev(self->cbLxOn, false);
	    }
	    if (LC0->UseLx) {
		DVal_Update((TFormRLC*)self->f_Lx, LC0->L * LC0->v[LC0->vi - 1]);
		TChkBox_SetChecked(self->cbLxOn, true);
	    } else {
		TChkBox_SetChecked(self->cbLxOn, false);
	    }
	    RePaint((TWins*)self->pnl);

	    switch (LC0->nr) {
	    case 1:
		DVal_Update((TFormRLC*)self->f_C2, LC0->C * LC0->v[i]);
		math_Filter(self);
		return true;
	    case 2:
		DVal_Update((TFormRLC*)self->f_C1, LC0->C * LC0->v[i++]);
		DVal_Update((TFormRLC*)self->f_L1, LC0->L * LC0->v[i++]);
		DVal_Update((TFormRLC*)self->f_C4, LC0->C * LC0->v[i]);
		math_Filter(self);
		return true;
	    case 3:
		DVal_Update((TFormRLC*)self->f_C1, LC0->C * LC0->v[i++]);
		DVal_Update((TFormRLC*)self->f_L1, LC0->L * LC0->v[i++]);
		DVal_Update((TFormRLC*)self->f_C2, LC0->C * LC0->v[i++]);
		DVal_Update((TFormRLC*)self->f_L3, LC0->L * LC0->v[i++]);
		DVal_Update((TFormRLC*)self->f_C4, LC0->C * LC0->v[i]);
		math_Filter(self);
		return true;
	    case 4:
		DVal_Update((TFormRLC*)self->f_C1, LC0->C * LC0->v[i++]);
		DVal_Update((TFormRLC*)self->f_L1, LC0->L * LC0->v[i++]);
		DVal_Update((TFormRLC*)self->f_C2, LC0->C * LC0->v[i++]);
		DVal_Update((TFormRLC*)self->f_L2, LC0->L * LC0->v[i++]);
		DVal_Update((TFormRLC*)self->f_C3, LC0->C * LC0->v[i++]);
		DVal_Update((TFormRLC*)self->f_L3, LC0->L * LC0->v[i++]);
		DVal_Update((TFormRLC*)self->f_C4, LC0->C * LC0->v[i]);
		math_Filter(self);
		return true;
	    }
	    return false;
	}
	return false;
}

/* ************** TP1 Ev ****************** */
static void cbXxOn_OnClick(TChkBox *self, long KeyState, bool DblClick) {
  TPage_TP1 *pg = self->subClass;
  TpcIni *pcIni = TpcIni_Create(appDat.inifile_app);
	XF_SaveOnOff((TPage_XFilter*)pg, pcIni);
	TpcIni_SaveAndFree(pcIni);
	RePaint((TWins*)pg->pnl);
	math_Filter(pg);
}

static void cbFxOn_OnClick(TChkBox *self, long KeyState, bool DblClick) {
  TPage_TP1 *pg = self->subClass;
//  TPage_Filter *pg = self->subClass;
  TpcIni *pcIni = TpcIni_Create(appDat.inifile_app);
	XF_SaveOnOff((TPage_XFilter*)pg, pcIni);
	TpcIni_SaveAndFree(pcIni);
	((TScale_pxy*)self->tag)->PaintDat = ((TChkBox*)self)->Checked;
	Refresh((TWins*)pg->pgfilt->xy_ind);
}

static void cbFxImpOn_OnClick(TChkBox *self, long KeyState, bool DblClick) {
  TPage_TP1 *pg = self->subClass;
  TpcIni *pcIni = TpcIni_Create(appDat.inifile_app);
	if (pg->cbF1ImpOn != self) TChkBox_SetChecked(pg->cbF1ImpOn, false);
	if (pg->cbF2ImpOn != self) TChkBox_SetChecked(pg->cbF2ImpOn, false);
	if (pg->cbF3ImpOn != self) TChkBox_SetChecked(pg->cbF3ImpOn, false);
	if (pg->cbF4ImpOn != self) TChkBox_SetChecked(pg->cbF4ImpOn, false);
	XF_SaveOnOff((TPage_XFilter*)pg, pcIni);
	TpcIni_SaveAndFree(pcIni);
	math_Filter(pg);
	Refresh((TWins*)pg->pgfilt->xy_ind);
}


static void pnl_OnPaint(TWins *self) {
  TPage_TP1 *pg = self->subClass;
  int fu, x;

	fu = XF_GetPaint_Fu((TPage_XFilter*)pg);
	x = pg->cbF1On->left + pg->cbF1On->width + 2;

	DrawSch(self->WinID, x, pg->cbF1On->top, fu);
	fu += 0x100;
	DrawSch(self->WinID, x, pg->cbF2On->top, fu);
	fu += 0x100;
	DrawSch(self->WinID, x, pg->cbF3On->top, fu);
	fu += 0x100;
	DrawSch(self->WinID, x, pg->cbF4On->top, fu);

	DrawPic(self->WinID, get_Pic_cOhm(), 
			pg->dv_Ra->left + pg->dv_Ra->width + 2,
			pg->dv_Ra->top + 5);

	str_toGraf(self, "pF", pg->dv_Cx->left + pg->dv_Cx->width + 1,
			       pg->dv_Cx->top + 3);

	str_toGraf(self, "uH", pg->dv_Lx->left + pg->dv_Lx->width + 1,
			       pg->dv_Lx->top + 3);

	str_toGraf(self, "pF", pg->dv_C4->left + pg->dv_C4->width + 1,
			       pg->dv_C4->top + 3);

	str_toGraf(self, "uH", pg->dv_L3->left + pg->dv_L3->width + 1,
			       pg->dv_L3->top + 3);
}

static void TP1_DVal_OnValChange(TDVal *dv, double val) {
	if (dv->owner) TDVal_DoValChange((TDVal*)dv->owner, val, tdvfu_ChangeEv);
}

static void TP1_fXxx_OnChange(TFormRLC* fRLC) {
	math_Filter((TPage_TP1*)fRLC->subClass);
}

#ifdef FILTER_PULS
static void Puls_OnChange(TFormPuls *self) {
	math_Filter((TPage_TP1*)self->subClass);
}

static void Border_lblPulsOnOff(TPage_TP1 *self) {
  int i;

	self->lblF1PulsOnOff->BorderFu &= ~mBorderL;
	self->lblF2PulsOnOff->BorderFu &= ~mBorderL;
	i = self->f_Puls->tag;
	if (i == 1) {
	    self->lblF1PulsOnOff->BorderFu |= mBorderL;
	} else if (i == 2) {
	    self->lblF2PulsOnOff->BorderFu |= mBorderL;
	}
	Refresh(self->lblF1PulsOnOff);
	Refresh(self->lblF2PulsOnOff);
}

static void lblPulsOnOff_OnClick(TWins *self, long KeyState, bool DblClick) {
  TPage_TP1 *pg = self->subClass;

	if (pg->f_Puls->Visible) {
	    if (self->tag == pg->f_Puls->tag) {
		pg->f_Puls->tag = 0;
		SetVisible((TWins*)pg->f_Puls, false);
	    } else {
		pg->f_Puls->tag = self->tag;
		Border_lblPulsOnOff(pg);
	    }
	} else {
	    pg->f_Puls->tag = self->tag;
	    SetVisible((TWins*)pg->f_Puls, true);
	}
	TFormPuls_SavePosOnOff(pg->f_Puls);
	math_Filter(pg);
}

static void Puls_OnCanClose(TFormPuls *self, int *CanClose) {
  TPage_TP1 *pg = self->subClass;

	if (self->CloseIsWM_DELETE) {
	    self->tag = 0;
	    TFormPuls_SavePosOnOff(self);
	    math_Filter(pg);
	}
//	pg->lblF2PulsOnOff->BorderFu &= ~mBorderL;
//	RePaint(pg->lblF2PulsOnOff);
	Border_lblPulsOnOff(pg);
}

static void Puls_OnShow(TFormPuls *self) {
  TPage_TP1 *pg = self->subClass;

	Border_lblPulsOnOff(pg);
/*
	if (self->tag == 1) {
	    pg->lblF1PulsOnOff->BorderFu |= mBorderL;
	    Refresh(pg->lblF1PulsOnOff);
	} else if (self->tag == 2) {
	    pg->lblF2PulsOnOff->BorderFu |= mBorderL;
	    Refresh(pg->lblF2PulsOnOff);
	}
*/
}
#endif /* FILTER_PULS */

/* static void TP1_OnRSync(TFormRLC* fRLC, int val) {
  TPage_TP1 *pg = fRLC->subClass;
	Form_R_sync(fRLC, pg->f_Rg, val);
	Form_R_sync(fRLC, pg->f_Rs, val);
	Form_R_sync(fRLC, pg->f_Rx, val);
	Form_R_sync(fRLC, pg->f_Ra, val);
} */
static void TP1_OnLSync(TFormRLC* fRLC, int val) {
  TPage_TP1 *pg = fRLC->subClass;
	Form_L_sync(fRLC, pg->f_L1, val);
	Form_L_sync(fRLC, pg->f_L2, val);
	Form_L_sync(fRLC, pg->f_L3, val);
}
static void TP1_OnCSync(TFormRLC* fRLC, int val) {
  TPage_TP1 *pg = fRLC->subClass;
	Form_C_sync(fRLC, pg->f_C1, val);
	Form_C_sync(fRLC, pg->f_C2, val);
	Form_C_sync(fRLC, pg->f_C3, val);
	Form_C_sync(fRLC, pg->f_C4, val);
}
/* ************** Save Load ****************** */
static void SaveData(TPage_TP1 *self, TpcIni *ini) {
	XF_SaveData((TPage_XFilter*)self, ini);

	self->f_L1->SaveData(self->f_L1, ini);
	self->f_L2->SaveData(self->f_L2, ini);
	self->f_L3->SaveData(self->f_L3, ini);

	self->f_C1->SaveData(self->f_C1, ini);
	self->f_C2->SaveData(self->f_C2, ini);
	self->f_C3->SaveData(self->f_C3, ini);
	self->f_C4->SaveData(self->f_C4, ini);
}

static void LoadData(TPage_TP1 *self, TpcIni *ini) {

	XF_LoadData((TPage_XFilter*)self, ini);

	self->f_L1->LoadData(self->f_L1, ini);
	self->f_L2->LoadData(self->f_L2, ini);
	self->f_L3->LoadData(self->f_L3, ini);

	self->f_C1->LoadData(self->f_C1, ini);
	self->f_C2->LoadData(self->f_C2, ini);
	self->f_C3->LoadData(self->f_C3, ini);
	self->f_C4->LoadData(self->f_C4, ini);

	math_Filter(self);
	Refresh((TWins*)self->pgfilt->xy_ind);
}

/* ************** TP1 Create ****************** */
static void DVal_Create_Xxx(TWins *Parent, TDVal **dv, TFormRLConoff **onoff, int *x, int y, int w, int nk, TPic *pic) {
  int tx = *x;
  TDVal *tdv = TDVal_CreateS(Parent, tx+19, y, w, 19);
	tdv->nk = nk;
	tdv->dv_fu = tdv_fu_LostFocusEnter;
	tdv->OnValChange = TP1_DVal_OnValChange;
	*dv = tdv;
	*onoff = TFormRLConoff_CreateU(Parent, tx, y+1, 18, 16, pic);
	*x = tx + w + 21;
}

/* static TFormR *Create_fR(TPage_TP1 *self, const char *Caption,
			const char *s, TDVal *dv, float val) {
  TFormR *fR = Create_Form_R(appDat.obj, Caption, self->gname, s, dv, val);
	fR->subClass = self;
	TDVal_Set_Value(dv, fR->R->dval);
	fR->OnChange = TP1_fXxx_OnChange;
	fR->OnSync = TP1_OnRSync;
	return fR;
} */
static TFormL *Create_fL(TPage_TP1 *self, const char *Caption,
			const char *s, TDVal *dv, float val) {
  TFormL *fL = Create_Form_L(appDat.obj, Caption, self->gname, s, dv, val);
	fL->subClass = self;
	TDVal_Set_Value(dv, fL->L->dval);
	fL->OnChange = TP1_fXxx_OnChange;
	fL->OnSync = TP1_OnLSync;
	return fL;
}
static TFormC *Create_fC(TPage_TP1 *self, const char *Caption,
			const char *s, TDVal *dv, float val) {
  TFormC *fC = Create_Form_C(appDat.obj, Caption, self->gname, s, dv, val);
	fC->subClass = self;
	TDVal_Set_Value(dv, fC->C->dval);
	fC->OnChange = TP1_fXxx_OnChange;
	fC->OnSync = TP1_OnCSync;
	return fC;
}

TPage_TP1 *page_Add_TP1(TPageControl *pagc, TPage_Filter *pgfilt) {
  TPage_TP1 *self;
  TWins *pnl;
  int x;
  TAddXFilter xf;

	xf.pagc = pagc;
	xf.pgfilt = pgfilt;
	xf.gname = "TP1";
	xf.psize = sizeof(TPage_TP1);
	xf.ini = get_appIni();
	xf.TDValOnValChange = TP1_DVal_OnValChange;
	xf.fXxx_OnChange = TP1_fXxx_OnChange;
	xf.OnRSync = NULL; //TP1_OnRSync;
	xf.OnLSync = TP1_OnLSync;
	xf.OnCSync = TP1_OnCSync;
	xf.dvfu = FU_RG|FU_RS|FU_RX|FU_RA|FU_CS|FU_CX|FU_LS|FU_LX;

	xf.cb_OnClick = cbXxOn_OnClick;
	xf.cbfu = FU_RS|FU_RX|FU_CS|FU_CX|FU_LS|FU_LX;
	xf.cbOnOff = 0xffe0;
	xf.cbY = 82;

	xf.cbFxOnClick = cbFxOn_OnClick;
	xf.cbFxImpOnClick = cbFxImpOn_OnClick;
	xf.cbF1Y = 105;
	xf.cbF2Y = 150;
	xf.cbF3Y = 195;
	xf.cbF4Y = 240;
	xf.cbFxfu = FU_F1|FU_F2|FU_F3|FU_F4;

	xf.LC0_Popupfu = FU_F1|FU_F2|FU_F3|FU_F4;

	self = (TPage_TP1*)PageAddXFilter(&xf);
	self->pnl->OnPaint = pnl_OnPaint;
	pnl = self->pnl;
/* C1 C2 C3 */
	x = 0;
	DVal_Create_Xxx(pnl, &self->dv_C1, &self->OnOff_C1, &x, 42, 54, 1, get_Pic_cC1());
	DVal_Create_Xxx(pnl, &self->dv_C2, &self->OnOff_C2, &x, 42, 54, 1, get_Pic_cC2());
	DVal_Create_Xxx(pnl, &self->dv_C3, &self->OnOff_C3, &x, 42, 54, 1, get_Pic_cC3());
	DVal_Create_Xxx(pnl, &self->dv_C4, &self->OnOff_C4, &x, 42, 54, 1, get_Pic_cC4());
/* L1 L2 L3 */
	x = 0;
	DVal_Create_Xxx(pnl, &self->dv_L1, &self->OnOff_L1, &x, 62, 54, 3, get_Pic_cL1());
	DVal_Create_Xxx(pnl, &self->dv_L2, &self->OnOff_L2, &x, 62, 54, 3, get_Pic_cL2());
	DVal_Create_Xxx(pnl, &self->dv_L3, &self->OnOff_L3, &x, 62, 54, 3, get_Pic_cL3());

	self->f_C1 = Create_fC(self, "TP1 C1", "C1", self->dv_C1, 8);
	self->f_C2 = Create_fC(self, "TP1 C2", "C2", self->dv_C2, 16);
	self->f_C3 = Create_fC(self, "TP1 C3", "C3", self->dv_C3, 16);
	self->f_C4 = Create_fC(self, "TP1 C4", "C4", self->dv_C4, 8);

	self->f_L1 = Create_fL(self, "TP1 L1", "L1", self->dv_L1, 0.027);
	self->f_L2 = Create_fL(self, "TP1 L2", "L2", self->dv_L2, 0.027);
	self->f_L3 = Create_fL(self, "TP1 L3", "L3", self->dv_L3, 0.027);

	TFormRLConoff_SetForm(self->OnOff_C1, (TForm*)self->f_C1);
	TFormRLConoff_SetForm(self->OnOff_C2, (TForm*)self->f_C2);
	TFormRLConoff_SetForm(self->OnOff_C3, (TForm*)self->f_C3);
	TFormRLConoff_SetForm(self->OnOff_C4, (TForm*)self->f_C4);
	TFormRLConoff_SetForm(self->OnOff_L1, (TForm*)self->f_L1);
	TFormRLConoff_SetForm(self->OnOff_L2, (TForm*)self->f_L2);
	TFormRLConoff_SetForm(self->OnOff_L3, (TForm*)self->f_L3);

	self->OnMsg = self_OnMsg;

	self->SaveData = SaveData;
	self->LoadData = LoadData;

#ifdef FILTER_PULS
	self->f_Puls = Create_Form_Puls(appDat.obj, "TP1 Puls", self->gname, "puls");
	self->f_Puls->subClass = self;
	self->f_Puls->OnChange = Puls_OnChange;
	self->f_Puls->OnCanClose = Puls_OnCanClose;
	self->f_Puls->OnShow = Puls_OnShow;

	self->lblF1PulsOnOff = TWins_CreateLabel(pnl, 280, 100, 50, 18, "F1 Puls", 0, mBorder);
	self->lblF1PulsOnOff->OnClick = lblPulsOnOff_OnClick;
	self->lblF1PulsOnOff->tag = 1;

	self->lblF2PulsOnOff = TWins_CreateLabel(pnl, 280, 150, 50, 18, "F2 Puls", 0, mBorder);
	self->lblF2PulsOnOff->OnClick = lblPulsOnOff_OnClick;
	self->lblF2PulsOnOff->tag = 2;
#endif /* FILTER_PULS */


	free_appIni(xf.ini);

	return self;
}
